"""Module for subtitle list panel

"""

from bisect import bisect
# from multiprocessing.spawn import old_main_modules
from PySide6.QtWidgets import QWidget, QPushButton, QScrollArea, QLineEdit, QTextEdit
from PySide6.QtGui import QColor, QPen, QPainter, QPolygonF, QTransform, QPainterPath, QFont, QFontMetrics, QPalette
from PySide6.QtCore import QMarginsF, QRectF, Qt, QPointF, Signal

from subtitld.modules import quality_check
from subtitld.modules import history
from subtitld.modules import subtitles
from subtitld.modules import utils
from subtitld.interface import subtitles_panel


class subtitles_panel_timeline_widget_timeline(QWidget):
    """Class for timeline QWidget"""
    seek = Signal(float)

    def __init__(widget, self):
        super().__init__()
        widget.w_waveform = 125
        widget.x_waveform = widget.width() - widget.w_waveform
        widget.subtitle_width = widget.width()
        widget.subtitle_x = 0
        widget.height_proportion = 1
        widget.show_limiters = False
        widget.subtitle_is_clicked = False
        widget.subtitle_start_is_clicked = False
        widget.subtitle_end_is_clicked = False
        widget.show_tug_of_war = False
        widget.tug_of_war_pressed = False
        widget.is_cursor_pressing = False
        widget.waveformsize = .7
        widget.offset = 0.0
        widget.mediaplayer_zoom = 100.0
        widget.main_self = self

        widget.show_editing_widgets = False

        widget.starting_time_qlineedit = QLineEdit(widget)
        widget.starting_time_qlineedit.editingFinished.connect(lambda: widget.qlineedit_editing_finished())
        widget.starting_time_qlineedit.setProperty('class', 'subtitles_panel_timeline_widget_qlineedit')

        widget.text_qtextedit = QTextEdit(widget)
        widget.text_qtextedit.setProperty('class', 'subtitles_panel_timeline_widget_qlineedit')
        widget.text_qtextedit.textChanged.connect(lambda: widget.qlineedit_editing_finished())
        widget.text_qtextedit.setAlignment(Qt.AlignCenter)

        widget.ending_time_qlineedit = QLineEdit(widget)
        widget.ending_time_qlineedit.editingFinished.connect(lambda: widget.qlineedit_editing_finished())
        widget.ending_time_qlineedit.setProperty('class', 'subtitles_panel_timeline_widget_qlineedit')

        widget.update_editing_widgets()

    def paintEvent(widget, event):
        """Function for paintEvent of Timeline"""
        painter = QPainter(widget)
        scroll_position = widget.main_self.subtitles_panel_timeline_widget.verticalScrollBar().value()
        scroll_height = widget.main_self.subtitles_panel_timeline_widget.height()

        painter.setRenderHint(QPainter.Antialiasing)

        waveform_background_qrectF = QRectF(
            widget.width() - widget.w_waveform,
            0,
            widget.w_waveform,
            widget.height()
        )

        painter.setPen(Qt.NoPen)
        painter.setBrush(QColor(widget.main_self.settings['timeline'].get('waveform_background_color', '#55153450')))

        painter.drawRect(waveform_background_qrectF)

        transform_rotate_90 = QTransform()
        transform_rotate_90.rotate(90)

        if widget.main_self.settings['timeline'].get('view_mode', 'verticalform') and widget.main_self.video_metadata.get('waveform', {}):
            if widget.main_self.settings['timeline'].get('view_mode', 'verticalform') in ['waveform', 'verticalform']:
                if widget.main_self.video_metadata.get('waveform', {}):
                    zoom_factor = 1
                    available_zoom = widget.mediaplayer_zoom
                    if available_zoom not in widget.main_self.video_metadata['waveform'].keys():
                        available_zoom = sorted(widget.main_self.video_metadata['waveform'].keys())[0]
                        zoom_factor = widget.mediaplayer_zoom / available_zoom

                    h_factor = (widget.main_self.video_metadata.get('duration', 0.01) * available_zoom) / len(widget.main_self.video_metadata['waveform'][available_zoom]['points'][0])

                    if widget.main_self.video_metadata['waveform'][available_zoom].get('qimages', []):
                        ypos = 0
                        for qimage in widget.main_self.video_metadata['waveform'][available_zoom]['qimages']:
                            wid = qimage.width() * h_factor * zoom_factor
                            if not ypos > scroll_position + scroll_height and not ypos + wid < scroll_position:
                                painter.drawImage(QRectF(widget.x_waveform, ypos, widget.w_waveform, wid), qimage.transformed(transform_rotate_90))
                            ypos += wid

                    elif widget.main_self.video_metadata['waveform'][available_zoom].get('points', []):
                        painter.setPen(QPen(QColor(widget.main_self.settings['timeline'].get('waveform_border_color', '#ff153450')), 1, Qt.SolidLine))
                        painter.setBrush(QColor(widget.main_self.settings['timeline'].get('waveform_fill_color', '#cc153450')))

                        x_position = 0
                        polygon = QPolygonF()

                        for point in widget.main_self.video_metadata['waveform'][available_zoom]['points'][0][int(scroll_position / (zoom_factor * h_factor)):int((scroll_position + scroll_height) / (zoom_factor * h_factor))]:
                            polygon.append(QPointF((x_position + scroll_position), widget.x_waveform + (widget.w_waveform * .5) + (point * (widget.waveformsize * 100))))
                            x_position += (zoom_factor * h_factor)

                        for point in reversed(widget.main_self.video_metadata['waveform'][available_zoom]['points'][1][int(scroll_position / (zoom_factor * h_factor)):int((scroll_position + scroll_height) / (zoom_factor * h_factor))]):
                            polygon.append(QPointF((x_position + scroll_position), widget.x_waveform + (widget.w_waveform * .5) + (point * (widget.waveformsize * 100))))
                            x_position -= (zoom_factor * h_factor)

                        painter.drawPolygon(polygon)

        if widget.main_self.subtitles_list:
            painter.setOpacity(1)

            for subtitle in sorted(widget.main_self.subtitles_list):
                if (subtitle[0] / widget.main_self.video_metadata.get('duration', 0.01)) > ((scroll_position + scroll_height) / widget.height()):
                    break
                elif (subtitle[0] + subtitle[1]) / widget.main_self.video_metadata.get('duration', 0.01) < (scroll_position / widget.height()):
                    continue
                else:
                    painter.setPen(Qt.NoPen)
                    if widget.main_self.selected_subtitle == subtitle:
                        # painter.setPen(QColor(widget.main_self.settings['timeline'].get('selected_subtitle_border_color', '#ff304251')))
                        painter.setBrush(QColor(widget.main_self.settings['timeline'].get('selected_subtitle_fill_color', '#cc3e5363')))
                    else:
                        # painter.setPen(QColor(widget.main_self.settings['timeline'].get('subtitle_border_color', '#ff6a7483')))
                        painter.setBrush(QColor(widget.main_self.settings['timeline'].get('subtitle_fill_color', '#ccb8cee0')))

                    subtitle_rect = QRectF(
                        widget.subtitle_x,
                        subtitle[0] * widget.height_proportion,
                        widget.subtitle_width,
                        subtitle[1] * widget.height_proportion,
                    )

                    painter.drawRect(subtitle_rect)

                    if not widget.main_self.selected_subtitle == subtitle or (widget.main_self.selected_subtitle == subtitle and not widget.show_editing_widgets):
                        start_time = utils.get_timeline_time_str(subtitle[0], ms=True)
                        start_time_width = QFontMetrics(QFont('Ubuntu', 8)).horizontalAdvance(start_time)
                        start_time_rect = QRectF(
                            subtitle_rect.left() + 20,
                            subtitle_rect.top(),
                            start_time_width + 20,
                            20
                        )

                        path = QPainterPath()
                        path.moveTo(start_time_rect.right() + 5, subtitle_rect.top())
                        path.lineTo(0, subtitle_rect.top())
                        path.lineTo(0, subtitle_rect.top() + 20)
                        path.lineTo(start_time_rect.right(), subtitle_rect.top() + 20)
                        path.lineTo(start_time_rect.right() + 5, subtitle_rect.top())
                        painter.drawPath(path)

                        painter.setFont(QFont('Ubuntu Mono', 8))
                        painter.setPen(QColor(widget.main_self.settings['timeline'].get('time_text_color', '#806a7483')))
                        painter.drawText(start_time_rect, Qt.AlignLeft | Qt.AlignVCenter, start_time)

                        approved, _, _ = quality_check.check_subtitle(subtitle, widget.main_self.settings['quality_check'])
                        if widget.main_self.settings['quality_check'].get('enabled', False) and not approved:
                            painter.setPen(QColor('#9e1a1a'))
                        elif widget.main_self.selected_subtitle == subtitle:
                            painter.setPen(QColor(widget.main_self.settings['timeline'].get('selected_subtitle_text_color', '#ffffffff')))
                        else:
                            painter.setPen(QColor(widget.main_self.settings['timeline'].get('subtitle_text_color', '#ff304251')))

                        text_rect = subtitle_rect - QMarginsF(22, 2, widget.w_waveform + 2, 2)
                        painter.setFont(QFont('Ubuntu', 10))
                        painter.drawText(text_rect, Qt.AlignVCenter | Qt.AlignLeft | Qt.TextWordWrap, subtitle[2])

                        if widget.show_limiters and (subtitle[1] * widget.height_proportion) > 40:
                            painter.setPen(Qt.NoPen)

                            path = QPainterPath()
                            path.moveTo(start_time_rect.right() + 5, subtitle_rect.top())
                            path.lineTo((widget.width() - widget.w_waveform) + 5, subtitle_rect.top())
                            path.lineTo((widget.width() - widget.w_waveform), subtitle_rect.top() + 20)
                            path.lineTo(start_time_rect.right(), subtitle_rect.top() + 20)
                            path.lineTo(start_time_rect.right() + 5, subtitle_rect.top())
                            painter.drawPath(path)

                            ly = 1
                            for _ in range(2):
                                if widget.main_self.selected_subtitle == subtitle:
                                    lpen = QPen(QColor('#07000000') if ly % 2 else QColor(widget.main_self.settings['timeline'].get('selected_subtitle_arrow_color', '#ff969696')), 2)
                                else:
                                    lpen = QPen(QColor('#07000000') if ly % 2 else QColor(widget.main_self.settings['timeline'].get('subtitle_arrow_color', '#ff969696')), 2)

                                painter.setPen(lpen)
                                painter.setBrush(Qt.NoBrush)
                                path = QPainterPath()
                                path.moveTo(start_time_rect.right() + ((widget.width() - widget.w_waveform - start_time_rect.right()) * .5) - 10, (subtitle_rect.top() + 10) + ly + 1)
                                path.lineTo(start_time_rect.right() + ((widget.width() - widget.w_waveform - start_time_rect.right()) * .5), (subtitle_rect.top() + 10) - (3) + ly + 1)
                                path.lineTo(start_time_rect.right() + ((widget.width() - widget.w_waveform - start_time_rect.right()) * .5) + 10, (subtitle_rect.top() + 10) + ly + 1)
                                painter.drawPath(path)
                                ly -= 1

                        painter.setPen(Qt.NoPen)

                        if widget.main_self.selected_subtitle == subtitle:
                            painter.setBrush(QColor(widget.main_self.settings['timeline'].get('selected_subtitle_fill_color', '#cc3e5363')))
                        else:
                            painter.setBrush(QColor(widget.main_self.settings['timeline'].get('subtitle_fill_color', '#ccb8cee0')))

                        end_time = utils.get_timeline_time_str(subtitle[0] + subtitle[1], ms=True)
                        end_time_width = QFontMetrics(QFont('Ubuntu', 8)).horizontalAdvance(end_time)
                        end_time_rect = QRectF(
                            subtitle_rect.left() + 20,
                            subtitle_rect.bottom() - 20,
                            end_time_width + 20,
                            20
                        )

                        path = QPainterPath()
                        path.moveTo(end_time_rect.right(), subtitle_rect.bottom() - 20)
                        path.lineTo(0, subtitle_rect.bottom() - 20)
                        path.lineTo(0, subtitle_rect.bottom())
                        path.lineTo(end_time_rect.right() + 5, subtitle_rect.bottom())
                        path.lineTo(end_time_rect.right(), subtitle_rect.bottom() - 20)
                        painter.drawPath(path)

                        painter.setFont(QFont('Ubuntu Mono', 8))
                        painter.setPen(QColor(widget.main_self.settings['timeline'].get('time_text_color', '#806a7483')))
                        painter.drawText(end_time_rect, Qt.AlignLeft | Qt.AlignVCenter, end_time)

                        approved, _, _ = quality_check.check_subtitle(subtitle, widget.main_self.settings['quality_check'])
                        if widget.main_self.settings['quality_check'].get('enabled', False) and not approved:
                            painter.setPen(QColor('#9e1a1a'))
                        elif widget.main_self.selected_subtitle == subtitle:
                            painter.setPen(QColor(widget.main_self.settings['timeline'].get('selected_subtitle_text_color', '#ffffffff')))
                        else:
                            painter.setPen(QColor(widget.main_self.settings['timeline'].get('subtitle_text_color', '#ff304251')))

                        text_rect = subtitle_rect - QMarginsF(22, 2, widget.w_waveform + 2, 2)
                        painter.setFont(QFont('Ubuntu', 10))
                        painter.drawText(text_rect, Qt.AlignVCenter | Qt.AlignLeft | Qt.TextWordWrap, subtitle[2])

                        if widget.show_limiters and (subtitle[1] * widget.height_proportion) > 40:
                            painter.setPen(Qt.NoPen)

                            path = QPainterPath()
                            path.moveTo(end_time_rect.right(), subtitle_rect.bottom() - 20)
                            path.lineTo((widget.width() - widget.w_waveform), subtitle_rect.bottom() - 20)
                            path.lineTo((widget.width() - widget.w_waveform) + 5, subtitle_rect.bottom())
                            path.lineTo(end_time_rect.right() + 5, subtitle_rect.bottom())
                            path.lineTo(end_time_rect.right(), subtitle_rect.bottom() - 20)
                            painter.drawPath(path)

                            ly = 0
                            for _ in range(2):
                                if widget.main_self.selected_subtitle == subtitle:
                                    lpen = QPen(QColor('#07000000') if ly % 2 else QColor(widget.main_self.settings['timeline'].get('selected_subtitle_arrow_color', '#ff969696')), 2)
                                else:
                                    lpen = QPen(QColor('#07000000') if ly % 2 else QColor(widget.main_self.settings['timeline'].get('subtitle_arrow_color', '#ff969696')), 2)

                                painter.setPen(lpen)
                                painter.setBrush(Qt.NoBrush)
                                path = QPainterPath()
                                path.moveTo(end_time_rect.right() + ((widget.width() - widget.w_waveform - end_time_rect.right()) * .5) - 10, (subtitle_rect.bottom() - 20 + 10) + ly - 2)
                                path.lineTo(end_time_rect.right() + ((widget.width() - widget.w_waveform - end_time_rect.right()) * .5), (subtitle_rect.bottom() - 20 + 10) + (3) + ly - 2)
                                path.lineTo(end_time_rect.right() + ((widget.width() - widget.w_waveform - end_time_rect.right()) * .5) + 10, (subtitle_rect.bottom() - 20 + 10) + ly - 2)
                                painter.drawPath(path)
                                ly += 1

                    #     if widget.main_self.selected_subtitle == subtitle:
                    #         painter.setPen(QColor(widget.main_self.settings['timeline'].get('selected_subtitle_arrow_color', '#ff969696')))
                    #     else:
                    #         painter.setPen(QColor(widget.main_self.settings['timeline'].get('subtitle_arrow_color', '#ff969696')))

                    #     painter.drawText(lim_rect, Qt.AlignCenter, '︿')

                    #     painter.setPen(Qt.NoPen)
                    #     lim_rect = QRectF(
                    #         widget.subtitle_x + 2,
                    #         (subtitle[0] * widget.height_proportion) + (subtitle[1] * widget.height_proportion) - 20,
                    #         widget.subtitle_width - 4,
                    #         18
                    #     )

                    #     painter.drawRect(lim_rect)

                    #     if widget.main_self.selected_subtitle == subtitle:
                    #         painter.setPen(QColor(widget.main_self.settings['timeline'].get('selected_subtitle_arrow_color', '#ff969696')))
                    #     else:
                    #         painter.setPen(QColor(widget.main_self.settings['timeline'].get('subtitle_arrow_color', '#ff969696')))

                    #     painter.drawText(lim_rect, Qt.AlignCenter, '﹀')

                    if widget.main_self.selected_subtitle == subtitle:
                        painter.setPen(QColor(widget.main_self.settings['timeline'].get('selected_subtitle_border_color', '#ff304251')))
                    else:
                        painter.setPen(QColor(widget.main_self.settings['timeline'].get('subtitle_border_color', '#ff6a7483')))

                    painter.drawLine(
                        int(widget.subtitle_x),
                        int(subtitle_rect.top()),
                        int(widget.width()),
                        int(subtitle_rect.top()),
                    )

                    painter.drawLine(
                        int(widget.subtitle_x),
                        int(subtitle_rect.bottom()),
                        int(widget.width()),
                        int(subtitle_rect.bottom()),
                    )

            painter.setOpacity(1)

        if bool(widget.show_tug_of_war):
            tug_of_war_pen = QPen(QColor(widget.main_self.settings['timeline'].get('selected_subtitle_arrow_color', '#ff969696')), 4, Qt.SolidLine, Qt.RoundCap)
            painter.setPen(tug_of_war_pen)
            ypos = int(widget.show_tug_of_war * widget.height_proportion) - 4
            x_tug_pos = 0
            for _ in range(6):
                painter.drawLine(
                    int(widget.subtitle_x + 8 + x_tug_pos),
                    ypos,
                    int(widget.subtitle_x + 8 + x_tug_pos),
                    ypos + 8
                )
                # painter.drawLine(ypos, widget.subtitle_y + 8 + x_tug_pos, ypos + 8, widget.subtitle_y + widget.subtitle_height - 8)
                x_tug_pos += (widget.subtitle_width - widget.w_waveform - 16) / 5

        if widget.main_self.player_widget.position is not None:
            painter.setPen(QPen(QColor(widget.main_self.settings['timeline'].get('cursor_color', '#ccff0000')), 2, Qt.SolidLine))
            cursor_pos = int(widget.main_self.player_widget.position * widget.height_proportion)
            painter.drawLine(0, cursor_pos, widget.width(), cursor_pos)

        painter.end()
        event.accept()

    def mousePressEvent(widget, event):
        """Function to call when mouse is pressed"""
        scroll_position = widget.main_self.subtitles_panel_timeline_widget.verticalScrollBar().value()
        scroll_height = widget.main_self.subtitles_panel_timeline_widget.height()

        cursor_is_out_of_view = bool(widget.main_self.player_widget.position * widget.height_proportion < widget.main_self.subtitles_panel_timeline_widget.verticalScrollBar().value() or widget.main_self.player_widget.position * widget.height_proportion > widget.main_self.subtitles_panel_timeline_widget.width() + widget.main_self.subtitles_panel_timeline_widget.verticalScrollBar().value())

        widget.is_cursor_pressing = True
        widget.main_self.selected_subtitle = False

        for subtitle in sorted(widget.main_self.subtitles_list):
            if (subtitle[0] / widget.main_self.video_metadata.get('duration', 0.01)) > ((scroll_position + scroll_height) / widget.height()):
                break
            elif (subtitle[0] + subtitle[1]) / widget.main_self.video_metadata.get('duration', 0.01) < (scroll_position / widget.height()):
                continue
            else:
                if (widget.subtitle_x < event.pos().x() < widget.width() - widget.w_waveform) and (((event.pos().y()) / widget.height_proportion) > subtitle[0] and ((event.pos().y()) / widget.height_proportion) < (subtitle[0] + subtitle[1])):
                    widget.main_self.selected_subtitle = subtitle
                    if event.pos().y() / widget.height_proportion > (subtitle[0] + subtitle[1]) - (20 / widget.height_proportion):
                        widget.subtitle_end_is_clicked = True
                        widget.offset = ((widget.main_self.selected_subtitle[0] + widget.main_self.selected_subtitle[1]) * widget.height_proportion) - event.pos().y()
                        widget.tug_of_war_pressed = widget.show_tug_of_war
                    else:
                        widget.offset = event.pos().y() - widget.main_self.selected_subtitle[0] * widget.height_proportion
                        if event.pos().y() / widget.height_proportion < subtitle[0] + (20 / widget.height_proportion):
                            widget.subtitle_start_is_clicked = True
                            widget.tug_of_war_pressed = widget.show_tug_of_war
                        else:
                            widget.subtitle_is_clicked = True
                    break

        if not (widget.subtitle_end_is_clicked or widget.subtitle_start_is_clicked or widget.subtitle_is_clicked) or cursor_is_out_of_view:
            widget.main_self.player_widget.position = (event.pos().y() / widget.height()) * widget.main_self.video_metadata.get('duration', 0.01)
            widget.seek.emit(widget.main_self.player_widget.position)

        if (widget.subtitle_is_clicked or widget.subtitle_start_is_clicked or widget.subtitle_end_is_clicked):
            history.history_append(widget.main_self.subtitles_list)

        widget.update()

    def mouseReleaseEvent(widget, event):
        """Function to call when mouse press is released"""
        widget.subtitle_is_clicked = False
        widget.subtitle_start_is_clicked = False
        widget.subtitle_end_is_clicked = False
        widget.is_cursor_pressing = False
        widget.tug_of_war_pressed = False
        widget.update()
        event.accept()

    def mouseMoveEvent(widget, event):
        widget.show_limiters = bool(event.pos().x() > widget.subtitle_x and event.pos().x() < (widget.subtitle_width - widget.w_waveform))

        for subtitle in sorted(widget.main_self.subtitles_list):
            last = widget.main_self.subtitles_list[widget.main_self.subtitles_list.index(subtitle) - 1] if widget.main_self.subtitles_list.index(subtitle) > 0 else [0, 0, '']
            nextsub = widget.main_self.subtitles_list[widget.main_self.subtitles_list.index(subtitle) + 1] if widget.main_self.subtitles_list.index(subtitle) < len(widget.main_self.subtitles_list) - 1 else [widget.main_self.video_metadata['duration'], 0, '']

            if subtitle[0] + subtitle[1] > event.pos().y() / widget.height_proportion > (subtitle[0] + subtitle[1]) - (4 / widget.height_proportion) and nextsub[0] < (subtitle[0] + subtitle[1] + .02) and widget.show_limiters:
                widget.show_tug_of_war = subtitle[0] + subtitle[1] + .0005
                break
            elif subtitle[0] < event.pos().y() / widget.height_proportion < subtitle[0] + (4 / widget.height_proportion) and last[0] + last[1] > subtitle[0] - .02 and widget.show_limiters:
                widget.show_tug_of_war = subtitle[0] - .0005
                break
            elif not widget.tug_of_war_pressed:
                widget.show_tug_of_war = False

        if widget.main_self.selected_subtitle:
            i = widget.main_self.subtitles_list.index(widget.main_self.selected_subtitle)
            last = widget.main_self.subtitles_list[widget.main_self.subtitles_list.index(widget.main_self.selected_subtitle) - 1] if widget.main_self.subtitles_list.index(widget.main_self.selected_subtitle) > 0 else [0, 0, '']
            nextsub = widget.main_self.subtitles_list[widget.main_self.subtitles_list.index(widget.main_self.selected_subtitle) + 1] if widget.main_self.subtitles_list.index(widget.main_self.selected_subtitle) < len(widget.main_self.subtitles_list) - 1 else [widget.main_self.video_metadata['duration'], 0, '']
            scenes_list = widget.main_self.video_metadata['scenes'] if len(widget.main_self.video_metadata['scenes']) > 1 else [0.0]
            scenes_list.append(widget.main_self.video_metadata['duration'])
            start_position = (event.pos().y() - widget.offset) / widget.height_proportion
            last_scene = scenes_list[bisect(scenes_list, start_position) - 1]
            next_scene = scenes_list[bisect(scenes_list, start_position)]

            if widget.subtitle_start_is_clicked:
                end = widget.main_self.subtitles_list[i][0] + widget.main_self.subtitles_list[i][1]
                if not start_position > (end - widget.main_self.settings['default_values'].get('minimum_subtitle_width', 1)):
                    if not (bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed) and round(last[0] + last[1] + .001, 3) == round(widget.main_self.selected_subtitle[0], 3)) and widget.main_self.settings['timeline'].get('snap', True) and widget.main_self.settings['timeline'].get('snap_limits', True) and (last[0] + last[1] + widget.main_self.settings['timeline'].get('snap_value', .1)) > start_position:
                        subtitles.move_start_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=last[0] + last[1] + 0.001, move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                    elif widget.main_self.settings['timeline'].get('snap', True) and widget.main_self.settings['timeline'].get('snap_grid', False):
                        if widget.main_self.settings['timeline'].get('grid_type', False) == 'frames':
                            difference = start_position % (1.0 / widget.main_self.video_metadata['framerate'])
                            subtitles.move_start_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, amount=difference, move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                        elif widget.main_self.settings['timeline'].get('grid_type', False) == 'seconds' and float(start_position) > float(float(int(start_position) + 1) - float(widget.main_self.settings['timeline'].get('snap_value', .1))):
                            subtitles.move_start_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=float(int(start_position) + 1), move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                        elif widget.main_self.settings['timeline'].get('grid_type', False) == 'seconds' and float(start_position) < float(float(int(start_position)) + float(widget.main_self.settings['timeline'].get('snap_value', .1))):
                            subtitles.move_start_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=float(int(start_position)), move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                        elif widget.main_self.settings['timeline'].get('grid_type', False) == 'scenes' and start_position > next_scene - widget.main_self.settings['timeline'].get('snap_value', .1):
                            subtitles.move_start_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=next_scene, move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                        elif widget.main_self.settings['timeline'].get('grid_type', False) == 'scenes' and start_position < last_scene + widget.main_self.settings['timeline'].get('snap_value', .1):
                            subtitles.move_start_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=last_scene, move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                        else:
                            subtitles.move_start_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=start_position, move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                    else:
                        subtitles.move_start_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=start_position, move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                if widget.tug_of_war_pressed:
                    widget.show_tug_of_war = widget.main_self.selected_subtitle[0]
            elif widget.subtitle_end_is_clicked:
                end_position = (event.pos().y() + widget.offset) / widget.height_proportion
                if not end_position < (widget.main_self.subtitles_list[i][0] + widget.main_self.settings['default_values'].get('minimum_subtitle_width', 1)):
                    if not (bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed) and round(end_position, 3) >= round(nextsub[0] - 0.001, 3)) and widget.main_self.settings['timeline'].get('snap', True) and widget.main_self.settings['timeline'].get('snap_limits', True) and (nextsub[0] - widget.main_self.settings['timeline'].get('snap_value', .1)) < end_position:
                        # widget.main_self.subtitles_list[i][1] = (nextsub[0] - 0.001) - widget.main_self.subtitles_list[i][0]
                        subtitles.move_end_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=(nextsub[0] - 0.001), move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                    elif widget.main_self.settings['timeline'].get('snap', True) and widget.main_self.settings['timeline'].get('snap_grid', False):
                        if widget.main_self.settings['timeline'].get('grid_type', False) == 'frames':
                            difference = end_position % (1.0 / widget.main_self.video_metadata['framerate'])
                            subtitles.move_end_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, amount=difference, move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                        elif widget.main_self.settings['timeline'].get('grid_type', False) == 'seconds' and float(end_position) > float(float(int(end_position) + 1) - float(widget.main_self.settings['timeline'].get('snap_value', .1))):
                            subtitles.move_end_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=float(int(end_position) + 1), move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                        elif widget.main_self.settings['timeline'].get('grid_type', False) == 'seconds' and float(end_position) < float(float(int(end_position)) + float(widget.main_self.settings['timeline'].get('snap_value', .1))):
                            subtitles.move_end_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=float(int(end_position)), move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                        elif widget.main_self.settings['timeline'].get('grid_type', False) == 'scenes' and end_position > next_scene - widget.main_self.settings['timeline'].get('snap_value', .1):
                            subtitles.move_end_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=next_scene, move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                        elif widget.main_self.settings['timeline'].get('grid_type', False) == 'scenes' and end_position < last_scene + widget.main_self.settings['timeline'].get('snap_value', .1):
                            subtitles.move_end_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=last_scene, move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                        else:
                            subtitles.move_end_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=end_position, move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                    else:
                        subtitles.move_end_subtitle(subtitles=widget.main_self.subtitles_list, selected_subtitle=widget.main_self.selected_subtitle, absolute_time=end_position, move_nereast=bool(widget.main_self.settings['timeline'].get('snap_move_nereast', False) or widget.tug_of_war_pressed))
                if widget.tug_of_war_pressed:
                    widget.show_tug_of_war = widget.main_self.selected_subtitle[0] + widget.main_self.selected_subtitle[1]
            elif widget.subtitle_is_clicked:
                if widget.main_self.settings['timeline'].get('snap', True) and widget.main_self.settings.get('timeline', {}).get('snap_moving', True) and ((nextsub[0] - widget.main_self.settings['timeline'].get('snap_value', .1)) < (start_position + widget.main_self.subtitles_list[i][1]) or (last[0] + last[1] + widget.main_self.settings['timeline'].get('snap_value', .1)) > start_position):
                    if (nextsub[0] - widget.main_self.settings['timeline'].get('snap_value', .1)) < (start_position + widget.main_self.subtitles_list[i][1]):
                        widget.main_self.subtitles_list[i][0] = nextsub[0] - widget.main_self.subtitles_list[i][1] - 0.001
                    else:
                        widget.main_self.subtitles_list[i][0] = last[0] + last[1] + 0.001
                elif widget.main_self.settings['timeline'].get('snap', True) and widget.main_self.settings['timeline'].get('snap_grid', False):
                    if widget.main_self.settings['timeline'].get('grid_type', False) == 'frames':
                        difference = start_position % (1.0 / widget.main_self.video_metadata['framerate'])
                        widget.main_self.subtitles_list[i][0] = start_position - difference
                    elif widget.main_self.settings['timeline'].get('grid_type', False) == 'seconds' and float(start_position) > float(float(int(start_position) + 1) - float(widget.main_self.settings['timeline'].get('snap_value', .1))):
                        widget.main_self.subtitles_list[i][0] = float(int(start_position) + 1)
                    elif widget.main_self.settings['timeline'].get('grid_type', False) == 'seconds' and float(start_position) < float(float(int(start_position)) + float(widget.main_self.settings['timeline'].get('snap_value', .1))):
                        widget.main_self.subtitles_list[i][0] = float(int(start_position))
                    elif widget.main_self.settings['timeline'].get('grid_type', False) == 'scenes' and start_position > next_scene - widget.main_self.settings['timeline'].get('snap_value', .1):
                        widget.main_self.subtitles_list[i][0] = next_scene
                    elif widget.main_self.settings['timeline'].get('grid_type', False) == 'scenes' and start_position < last_scene + widget.main_self.settings['timeline'].get('snap_value', .1):
                        widget.main_self.subtitles_list[i][0] = last_scene
                    else:
                        widget.main_self.subtitles_list[i][0] = start_position
                else:
                    widget.main_self.subtitles_list[i][0] = start_position

        if widget.is_cursor_pressing and not (widget.subtitle_start_is_clicked or widget.subtitle_end_is_clicked or widget.subtitle_is_clicked):
            widget.seek.emit((event.pos().y() / widget.height()) * widget.main_self.video_metadata.get('duration', 0.01))

        widget.update()

    def mouseDoubleClickEvent(widget, event):
        widget.show_editing_widgets = True
        widget.update_editing_widgets()
        event.accept()

    def resizeEvent(widget, event):
        """Function to call when timeline is resized"""
        # widget.height_proportion = widget.width()/widget.main_self.video_metadata.get('duration', 0.01)
        widget.x_waveform = widget.width() - widget.w_waveform
        widget.subtitle_width = widget.width()
        widget.height_proportion = widget.height() / widget.main_self.video_metadata.get('duration', 0.01)

    def leaveEvent(widget, event):
        widget.qlineedit_editing_finished()
        widget.show_editing_widgets = False
        widget.update_editing_widgets()
        event.accept()

    def qlineedit_editing_finished(widget):
        if widget.show_editing_widgets and widget.main_self.selected_subtitle and widget.starting_time_qlineedit.text() and widget.ending_time_qlineedit.text():
            widget.main_self.selected_subtitle[0] = float(widget.starting_time_qlineedit.text())
            widget.main_self.selected_subtitle[1] = float(widget.ending_time_qlineedit.text()) - widget.main_self.selected_subtitle[0]
            widget.main_self.selected_subtitle[2] = widget.text_qtextedit.toPlainText()
        widget.update()

    def update_editing_widgets(widget):
        if widget.main_self.selected_subtitle:
            start_time = utils.get_timeline_time_str(widget.main_self.selected_subtitle[0], ms=True)
            start_time_width = QFontMetrics(QFont('Ubuntu', 8)).horizontalAdvance(start_time)

            widget.starting_time_qlineedit.setGeometry(
                int(widget.subtitle_x) + 10,
                int(widget.main_self.selected_subtitle[0] * widget.height_proportion) + 3,
                start_time_width + 20,
                15,
            )
            widget.starting_time_qlineedit.setText(str(widget.main_self.selected_subtitle[0]))
            widget.starting_time_qlineedit.setVisible(True)

            widget.text_qtextedit.setGeometry(
                int(widget.subtitle_x) + 10,
                int((widget.main_self.selected_subtitle[0]) * widget.height_proportion) + 24,
                widget.x_waveform - 20,
                int((widget.main_self.selected_subtitle[1]) * widget.height_proportion) - 46
            )
            widget.text_qtextedit.blockSignals(True)
            widget.text_qtextedit.setText(str(widget.main_self.selected_subtitle[2]))
            widget.text_qtextedit.blockSignals(False)

            widget.text_qtextedit.setVisible(True)

            end_time = utils.get_timeline_time_str(widget.main_self.selected_subtitle[0], ms=True)
            end_time_width = QFontMetrics(QFont('Ubuntu', 8)).horizontalAdvance(end_time)

            widget.ending_time_qlineedit.setGeometry(
                int(widget.subtitle_x) + 10,
                int((widget.main_self.selected_subtitle[0] + widget.main_self.selected_subtitle[1]) * widget.height_proportion) - 17,
                end_time_width + 20,
                15,
            )
            widget.ending_time_qlineedit.setText(str(widget.main_self.selected_subtitle[0] + widget.main_self.selected_subtitle[1]))
            widget.ending_time_qlineedit.setVisible(True)

        widget.starting_time_qlineedit.setVisible(widget.show_editing_widgets)
        widget.text_qtextedit.setVisible(widget.show_editing_widgets)
        widget.ending_time_qlineedit.setVisible(widget.show_editing_widgets)


def add_button(self):
    self.subtitles_panel_widget_button_timeline = QPushButton()
    self.subtitles_panel_widget_button_timeline.setObjectName('subtitles_panel_widget_button_timeline')
    self.subtitles_panel_widget_button_timeline.setProperty('class', 'subtitles_panel_left_button')
    self.subtitles_panel_widget_button_timeline.setCheckable(True)
    self.subtitles_panel_widget_button_timeline.setFixedWidth(23)
    # self.subtitles_panel_widget_button_timeline.icon().addPixmap(self.subtitles_panel_widget_button_timeline.icon(), QIcon.Disable)
    self.subtitles_panel_widget_button_timeline.clicked.connect(lambda vision: subtitles_panel.update_subtitles_panel_widget_vision(self, 'timeline'))
    self.subtitles_panel_widget_buttons_vbox.addWidget(self.subtitles_panel_widget_button_timeline)


def add_widgets(self):
    self.subtitles_panel_timeline_widget = QScrollArea()
    self.subtitles_panel_timeline_widget.setObjectName('subtitles_panel_timeline_widget')
    self.subtitles_panel_timeline_widget.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
    self.subtitles_panel_timeline_widget.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

    self.subtitles_panel_timeline_widget_timeline = subtitles_panel_timeline_widget_timeline(self)
    self.subtitles_panel_timeline_widget_timeline.setMouseTracking(True)
    self.subtitles_panel_timeline_widget_timeline.seek.connect(lambda position: subtitles_panel_timeline_widget_timeline_seek(self, position))
    # self.subtitles_panel_timeline_widget_timeline.focusout.connect(lambda: subtitles_panel_timeline_widget_timeline_loses_focus(self))
    # self.subtitles_panel_timeline_widget_timeline.doubleclicked.connect(lambda: subtitles_panel_timeline_widget_timeline_doubleclicked(self))
    self.subtitles_panel_timeline_widget.setWidget(self.subtitles_panel_timeline_widget_timeline)

    self.subtitles_panel_stackedwidgets.addWidget(self.subtitles_panel_timeline_widget)


def update_subtitles_panel_timeline(self):
    # print(self.video_metadata.get('duration', 0.01) * self.mediaplayer_zoom)
    self.subtitles_panel_timeline_widget_timeline.update()

    if self.settings['timeline'].get('scrolling', 'page') == 'follow':
        if (self.player_widget.position * (self.subtitles_panel_timeline_widget_timeline.height() / self.video_metadata.get('duration', 0.01))) > self.subtitles_panel_timeline_widget.width() * .5 and (self.player_widget.position * (self.subtitles_panel_timeline_widget_timeline.height() / self.video_metadata.get('duration', 0.01))) < (self.subtitles_panel_timeline_widget_timeline.height() - (self.subtitles_panel_timeline_widget.height() * .5)):
            update_scrollbar(self, position='middle')
    elif self.settings['timeline'].get('scrolling', 'page') == 'page' and (self.player_widget.position * (self.subtitles_panel_timeline_widget_timeline.height() / self.video_metadata.get('duration', 0.01))) > self.subtitles_panel_timeline_widget.height() + self.subtitles_panel_timeline_widget.verticalScrollBar().value():
        update_scrollbar(self)

    # self.subtitles_panel_timeline_widget_timeline.setGeometry(0, 0, self.subtitles_panel_subtitles_panel_timeline_widget_timeline.height(), self.video_metadata.get('duration', 0.01) * self.mediaplayer_zoom)
    # print(self.subtitles_panel_timeline_widget_timeline.height())


def update_scrollbar(self, position=0):
    """Function to update scrollbar of timeline"""
    if position == 'middle':
        offset = self.subtitles_panel_timeline_widget.height() * .5
    elif isinstance(position, float):
        offset = self.subtitles_panel_timeline_widget.height() * position
    elif isinstance(position, int):
        offset = position
    # self.subtitles_panel_timeline_widget.verticalScrollBar().setValue(self.player_widget.mpv.time_pos * (self.timeline_widget.width()/self.video_metadata.get('duration', 0.01)) - offset)
    self.subtitles_panel_timeline_widget.verticalScrollBar().setValue(int(self.player_widget.position * (self.subtitles_panel_timeline_widget_timeline.height() / self.video_metadata.get('duration', 0.01)) - offset))


def timeline_resized(self):
    self.subtitles_panel_timeline_widget_timeline.setGeometry(0, 0, self.subtitles_panel_timeline_widget.width(), int(self.video_metadata.get('duration', 0.01) * self.mediaplayer_zoom))


def subtitles_panel_timeline_widget_timeline_seek(self, position):
    self.player_widget.seek(position)



import os
from mpv import MPV, MpvRenderContext, MpvGlGetProcAddressFn

from PySide6.QtWidgets import QWidget, QVBoxLayout, QLabel, QGraphicsOpacityEffect
from PySide6.QtCore import Signal, Qt, QRect, QPropertyAnimation, QEasingCurve, QMargins
from PySide6.QtGui import QPainter, QPen, QColor, QFont, QBrush
from PySide6.QtOpenGLWidgets import QOpenGLWidget

from subtitld.modules.utils import GetProcAddressGetter
from subtitld.interface import playercontrols, subtitles_panel


class MpvWidget(QOpenGLWidget):
    """Main MPV widget class"""
    positionChanged = Signal(float, int)
    eofReached = Signal()

    def __init__(widget, parent=None):
        super().__init__(parent)

        widget.mpv = MPV(
            # ytdl=False,
            loglevel='info',
            log_handler=print
        )

        widget.mpv_gl = None
        widget.get_proc_addr_c = MpvGlGetProcAddressFn(GetProcAddressGetter().wrap)
        widget.frameSwapped.connect(
            widget.swapped, Qt.ConnectionType.DirectConnection
        )

        for key, value in {
            # "config": False,
            'osd_level': 0,
            'sub_auto': False,
            'sub_ass': False,
            'sub_visibility': False,
            'keep_open': True,
            'cursor_autohide': False,
            'input_cursor': False,
            'input_default_bindings': False,
            'stop_playback_on_init_failure': False,
            'audio_file_auto': False,
            'input_vo_keyboard': False,
            'sid': False,
            "quiet": True,
            "msg-level": "all=info",
            "osc": False,
            "osd-bar": False,
            "input-cursor": False,
            "input-vo-keyboard": False,
            "input-default-bindings": False,
            # "ytdl": False,
            "sub-auto": False,
            "audio-file-auto": False,
            "vo": "libmpv",
            "hwdec": "auto",
            "pause": True,
            "idle": True,
            "blend-subtitles": "video",
            "video-sync": "display-vdrop",
            "keepaspect": True,
            "stop-playback-on-init-failure": False,
            "keep-open": True,
            # "track-auto-selection": False,
        }.items():
            setattr(widget.mpv, key, value)

        widget.position = 0.0
        widget.mpv.observe_property('time-pos', widget.position_changed)
        widget.mpv.observe_property('eof-reached', widget.eof_reached)

    def initializeGL(widget):
        widget.mpv_gl = MpvRenderContext(
            widget.mpv,
            api_type="opengl",
            opengl_init_params={"get_proc_address": widget.get_proc_addr_c},
        )
        widget.mpv_gl.update_cb = widget.on_update

    def paintGL(widget):
        if widget.mpv_gl:
            ratio = widget.devicePixelRatioF()
            w = int(widget.width() * ratio)
            h = int(widget.height() * ratio)
            fbo = widget.defaultFramebufferObject()
            widget.mpv_gl.render(
                flip_y=True,
                opengl_fbo={
                    "fbo": fbo,
                    "w": w,
                    "h": h,
                },
            )

    # @Slot()
    # def maybe_update(widget):
    #     """Maybeupdate function"""
    #     if widget.window().isMinimized():
    #         widget.makeCurrent()
    #         widget.paintGL()
    #         widget.context().swapBuffers(widget.context().surface())
    #         widget.swapped()
    #         widget.doneCurrent()
    #     else:
    #         widget.update()

    def on_update(widget, ctx=None):
        widget.update()
        #  print(widget.width())
        #  print(widget.height())

    def on_update_fake(widget, ctx=None):
        pass

    def swapped(widget):
        if widget.mpv_gl:
            widget.mpv_gl.report_swap()

    def closeEvent(widget, _):
        widget.makeCurrent()
        if widget.mpv_gl:
            widget.mpv_gl.update_cb = widget.on_update_fake
            widget.mpv_gl.free()

    def position_changed(widget, _, pos):
        """Position changed function. It calls update timeline paint."""
        if pos:
            widget.position = pos
        if pos is not None:
            widget.positionChanged.emit(pos, 1)
            #  widget.parent.parent().timeline.update(widget.parent.parent())

    def eof_reached(widget, _, property):
        widget.eofReached.emit()

    def loadfile(widget, filepath) -> None:
        """Function to load a media file"""
        if os.path.isfile(filepath):
            widget.mpv.command('loadfile', filepath, 'replace')
            widget.mpv.wait_for_property('seekable')
        widget.mpv.pause = True

    def frameStep(widget) -> None:
        """Function to move forward one step (frame)"""
        widget.mpv.command('frame-step')

    def frameBackStep(widget) -> None:
        """Function to move backward one step (frame)"""
        widget.mpv.command('frame-back-step')

    def seek(widget, pos=0.0, method='absolute+exact') -> None:
        """Function to seek at some position"""
        widget.mpv.seek(pos, method)
        widget.position = pos

    def stop(widget) -> None:
        """Function to stop playback (fake stop, it is pause + position 0)"""
        widget.mpv.pause = True
        # print(dir(widget.mpv))
        if widget.mpv.filename:
            widget.position = 0.0
            widget.seek()

    def pause(widget) -> None:
        """Function to pause playback (fake pause, it just changes actual playback status)"""
        widget.mpv.pause = not widget.mpv.pause

    def play(widget) -> None:
        """Function to play (fake play, it just changes actual playback status)"""
        if widget.mpv.pause:
            widget.mpv.pause = False

    def mute(widget) -> None:
        """Function to mute"""
        widget.property('mute', not widget.property('mute'))

    def volume(widget, vol: int) -> None:
        """Function to change volume"""
        widget.property('volume', vol)

    # def resizeEvent(widget, event):
    #    event.accept()
    #    #print(widget.width())
    #    #print(widget.height())


class PlayerSubtitleLayer(QLabel):
    """Lass of subtitle layer"""
    def __init__(widget, parent=None):
        super().__init__(parent)
        widget.subtitle_text = ''
        widget.action_safe_margin = .9
        widget.title_safe_margin = .8
        widget.style = {}

    def paintEvent(widget, event):
        """Function to paint subtitle layer"""
        painter = QPainter(widget)
        painter.setRenderHint(QPainter.Antialiasing)

        painter.setFont(QFont(widget.style.get('font_family', 'Ubuntu'), widget.style.get('font_size', 40)))

        title_safe_margin_qrect = widget.rect() - QMargins(
            (widget.style.get('safe_margin_title_x', 10) / 100) * widget.width(),
            (widget.style.get('safe_margin_title_y', 10) / 100) * widget.height(),
            (widget.style.get('safe_margin_title_x', 10) / 100) * widget.width(),
            (widget.style.get('safe_margin_title_y', 10) / 100) * widget.height()
        )

        # QRect(
        #     widget.width() * (widget.style.get('safe_margin_title_x', 10) / 100.0),
        #     widget.height() * (widget.style.get('safe_margin_title_y', 10) / 100.0),
        #     widget.width() * (1.0 - ((widget.style.get('safe_margin_title_x', 10) * 2) / 100)),
        #     widget.height() * (1.0 - ((widget.style.get('safe_margin_title_y', 10) * 2) / 100))
        # )

        if widget.subtitle_text:

            if widget.style.get('backgroundbox_enabled', True):
                text_rect = painter.boundingRect(title_safe_margin_qrect, Qt.AlignBottom | Qt.AlignHCenter | Qt.TextWordWrap, widget.subtitle_text)

                painter.setPen(Qt.NoPen)
                painter.setBrush(QBrush(QColor(widget.style.get('backgroundbox_color', '#55000000'))))

                if widget.style.get('backgroundbox_border_radius', 5):
                    painter.drawRoundedRect(text_rect.marginsAdded(QMargins(widget.style.get('backgroundbox_padding', 10), widget.style.get('backgroundbox_padding', 10), widget.style.get('backgroundbox_padding', 10), widget.style.get('backgroundbox_padding', 10))), widget.style.get('backgroundbox_border_radius', 5), widget.style.get('backgroundbox_border_radius', 5))
                else:
                    painter.drawRect(text_rect.marginsAdded(QMargins(widget.style.get('backgroundbox_padding', 10), widget.style.get('backgroundbox_padding', 10), widget.style.get('backgroundbox_padding', 10), widget.style.get('backgroundbox_padding', 10))))

                painter.setBrush(Qt.NoBrush)

            if widget.style.get('shadow_enabled', True):
                painter.setPen(QPen(widget.style.get('shadow_color', '#ff000000')))
                painter.drawText(title_safe_margin_qrect - QMargins(widget.style.get('shadow_x', 2), widget.style.get('shadow_y', 2), -widget.style.get('shadow_x', 2), -widget.style.get('shadow_y', 2)), Qt.AlignHCenter | Qt.AlignBottom | Qt.TextWordWrap, widget.subtitle_text)

            painter.setPen(QPen(widget.style.get('color', '#ffffffff')))
            painter.drawText(title_safe_margin_qrect, Qt.AlignHCenter | Qt.AlignBottom | Qt.TextWordWrap, widget.subtitle_text)

        if widget.style.get('safe_margin_action_enabled', False):
            action_safe_margin_qrect = widget.rect() - QMargins(
                (widget.style.get('safe_margin_action_x', 5) / 100) * widget.width(),
                (widget.style.get('safe_margin_action_y', 5) / 100) * widget.height(),
                (widget.style.get('safe_margin_action_x', 5) / 100) * widget.width(),
                (widget.style.get('safe_margin_action_y', 5) / 100) * widget.height()
            )

            painter.setPen(QPen(QColor(widget.style.get('safe_margin_action_color', '#dd67FF4D')), 1, Qt.SolidLine))
            painter.drawRect(action_safe_margin_qrect)

            painter.drawLine(
                widget.width() * .5,
                action_safe_margin_qrect.y(),
                widget.width() * .5,
                action_safe_margin_qrect.y() + (widget.height() * .025)
            )
            painter.drawLine(
                widget.width() * .5,
                action_safe_margin_qrect.y() + action_safe_margin_qrect.height(),
                widget.width() * .5,
                action_safe_margin_qrect.y() + action_safe_margin_qrect.height() - (widget.height() * .025)
            )
            painter.drawLine(
                action_safe_margin_qrect.x(),
                widget.height() * .5,
                action_safe_margin_qrect.x() + (widget.width() * .025),
                widget.height() * .5
            )
            painter.drawLine(
                action_safe_margin_qrect.x() + action_safe_margin_qrect.width(),
                widget.height() * .5,
                action_safe_margin_qrect.x() + action_safe_margin_qrect.width() - (widget.width() * .025),
                widget.height() * .5
            )

        if widget.style.get('safe_margin_title_enabled', False):
            painter.setPen(QPen(QColor(widget.style.get('safe_margin_title_color', '#ddff0000')), 1, Qt.SolidLine))
            painter.drawRect(title_safe_margin_qrect)

        painter.end()
        event.accept()

    def setSubtitleText(widget, text):
        """Function to change subtitle layer text"""
        widget.subtitle_text = text

    def update_style(widget, style):
        widget.style = style
        widget.update()


class PlayerWidgetArea(QWidget):
    """Class to reimplement enter and leave events on subtitle layer"""
    def enterEvent(self, event):
        """Function to call when cursor enter player widget area"""
        event.accept()

    def leaveEvent(self, event):
        """Function to call when cursor leave player widget area"""
        event.accept()


def load(self):
    """Function to load player widgets"""

    # self.layer_player = QWidget(self)
    # self.layer_player.setLayout(QVBoxLayout())
    # self.layer_player.setContentsMargins(self.subtitles_panel_width_proportion * self.width(), 20, 20, 200)
    # self.layer_player.layout().setSpacing(0)

    # self.layer_player_left_spacer = QWidget()
    # self.layer_player_left_spacer.setMaximumWidth(0)
    # # sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Preferred)
    # # self.layer_player_left_spacer.setSizePolicy(sizePolicy)
    # self.layer_player_left_spacer_animation = QPropertyAnimation(self.layer_player_left_spacer, b'maximumWidth')
    # self.layer_player_left_spacer_animation.setEasingCurve(QEasingCurve.OutCirc)
    # self.layer_player_hbox.addWidget(self.layer_player_left_spacer)

    # self.layer_player_vbox = QVBoxLayout()
    # self.layer_player_vbox.setContentsMargins(0, 0, 0, 0)

    self.videoinfo_label = QLabel()  # parent=self.layer_player
    # self.videoinfo_label.setObjectName('videoinfo_label')
    # self.videoinfo_label.setFixedHeight(20)
    # sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
    # self.videoinfo_label.setSizePolicy(sizePolicy)
    # layer_player.layout().addWidget(self.videoinfo_label)

    self.player_widget = MpvWidget()
    # sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
    # sizePolicy.setWidthForHeight(self.player_widget.sizePolicy().hasWidthForHeight())
    # self.player_widget.setSizePolicy(sizePolicy)
    # self.player_widget_transparency = QGraphicsOpacityEffect()
    # self.player_widget.setGraphicsEffect(self.player_widget_transparency)
    # self.player_widget_transparency_animation = QPropertyAnimation(self.player_widget_transparency, b'opacity')
    # self.player_widget_transparency_animation.setEasingCurve(QEasingCurve.OutExpo)
    # self.player_widget_transparency.setOpacity(1)
    # self.player_widget_animation = QPropertyAnimation(self.player_widget, b'geometry')
    # self.player_widget_animation.setEasingCurve(QEasingCurve.OutCirc)
    self.player_widget.positionChanged.connect(lambda: update_timelines(self))
    self.player_widget.eofReached.connect(lambda: eof_reached(self))
    self.player_widget.setLayout(QVBoxLayout(self.player_widget))

    self.player_subtitle_layer = PlayerSubtitleLayer()
    self.player_subtitle_layer.setWordWrap(True)
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])
    self.player_subtitle_layer.setObjectName('player_subtitle_layer')
    self.player_widget.layout().addWidget(self.player_subtitle_layer)

    # class player_border(QLabel):
    #     def resizeEvent(widget, e):
    #         if self.video_metadata:
    #             aspect_ratio = self.video_metadata.get('height', 1080) / self.video_metadata.get('width', 1920)
    #             widget.setFixedHeight(widget.height() * aspect_ratio) #self.video_metadata.get('width', 1920), self.video_metadata.get('height', 1080))
    #            # w = widget.width()
    #            # h = widget.height()
    #            # if w / h > aspect_ratio:  # too wide
    #            #     widget.setFixedHeight(h)
    #            #     widget.layout().setDirection(QBoxLayout.LeftToRight)
    #            #     widget_stretch = h * aspect_ratio
    #            #     outer_stretch = (w - widget_stretch) / 2 + 0.5
    #            # else:  # too tall
    #            #     widget.layout().setDirection(QBoxLayout.TopToBottom)
    #            #     widget_stretch = w / aspect_ratio
    #            #     outer_stretch = (h - widget_stretch) / 2 + 0.5

    self.player_border = QLabel(self)  # parent=self.layer_player
    self.player_border_transparency = QGraphicsOpacityEffect()
    self.player_border.setGraphicsEffect(self.player_border_transparency)
    self.player_border_transparency_animation = QPropertyAnimation(self.player_border_transparency, b'opacity')
    self.player_border_transparency_animation.setEasingCurve(QEasingCurve.OutCirc)
    self.player_border_transparency.setOpacity(0)
    self.player_border_animation = QPropertyAnimation(self.player_border, b'geometry')
    self.player_border_animation.setEasingCurve(QEasingCurve.OutCirc)

    self.player_border.setObjectName('player_border')
    self.player_border.setLayout(QVBoxLayout(self.player_border))
    self.player_border.layout().setContentsMargins(1, 1, 1, 1)
    self.player_border.layout().addWidget(self.player_widget)

    # self.layer_player.layout().addWidget(self.player_border)

    # class player_ratio_widget(QLabel):
    #     def __init__(widget, child):
    #         super().__init__()
    #         widget.aspect_ratio = 1.77777#child.size().width() / child.size().height()
    #         widget.setLayout(QBoxLayout(QBoxLayout.LeftToRight))
    #         #  add spacer, then widget, then spacer
    #         widget.layout().addItem(QSpacerItem(0, 0))
    #         widget.layout().addWidget(child)
    #         widget.layout().addItem(QSpacerItem(0, 0))
    #         #sizePolicy = QSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
    #         #widget.setSizePolicy(sizePolicy)

    #     def resizeEvent(widget, e):
    #         w = e.size().width()
    #         h = e.size().height()
    #         if w / h > widget.aspect_ratio:  # too wide
    #             widget.layout().setDirection(QBoxLayout.LeftToRight)
    #             widget_stretch = h * widget.aspect_ratio
    #             outer_stretch = (w - widget_stretch) / 2 + 0.5
    #         else:  # too tall
    #             widget.layout().setDirection(QBoxLayout.TopToBottom)
    #             widget_stretch = w / widget.aspect_ratio
    #             outer_stretch = (h - widget_stretch) / 2 + 0.5

    #         widget.layout().setStretch(0, outer_stretch)
    #         widget.layout().setStretch(1, widget_stretch)
    #         widget.layout().setStretch(2, outer_stretch)

    # self.player_ratio_widget = player_ratio_widget(child=self.player_border)#QLabel()

    # self.layer_player.layout().addWidget(self.player_ratio_widget)

    # self.layer_player_hbox.addLayout(self.layer_player_vbox)


def update(self):
    """Function to update player widgets"""
    # self.layer_player.setVisible(bool(self.video_metadata))
    # self.player_border.setVisible(bool(self.video_metadata))
    update_safety_margins_subtitle_layer(self)


def resized(self):
    """Function to resize player widgets"""
    # self.layer_player.setGeometry(0, 0, self.width(), self.height())
    # self.layer_player.layout().setContentsMargins(self.width()*self.subtitles_panel_width_proportion, 20, 20, 200)
    # TODO
    # self.layer_player_left_spacer.setMaximumWidth(self.width()*self.subtitles_panel_width_proportion)
    # self.player_widget_area.setGeometry(0, 0, self.width(), self.height()-self.playercontrols_widget.height())
    # self.videoinfo_label.setGeometry(self.player_widget_area.x(), 20, self.player_widget_area.width(), 50)

    resize_player_widget(self)


def update_safety_margins_subtitle_layer(self):
    """Function to update margins preferences"""
    self.player_subtitle_layer.show_action_safe_margin = self.settings['safety_margins'].get('show_action_safe_margins', False)
    self.player_subtitle_layer.show_title_safe_margin = self.settings['safety_margins'].get('show_title_safe_margins', False)
    self.player_subtitle_layer.update()

# def player_subtitle_textedit_changed(self):
#     old_selected_subtitle = self.selected_subtitle
#     counter = self.subtitles_list.index(old_selected_subtitle)
#     self.subtitles_list[counter][2] = self.player_subtitle_textedit.toPlainText()
#     self.subtitles_panel.update_subtitles_panel_qlistwidget(self)
#     self.timeline.update(self)
#     update_subtitle_layer(self)


def eof_reached(self):
    self.player_widget.mpv.pause = True
    self.playercontrols_playpause_button.setChecked(False)
    playercontrols.playercontrols_playpause_button_update(self)


def update_speed(self):
    """Function to change playback speed"""
    self.player_widget.mpv.speed = self.playback_speed if self.change_playback_speed.isChecked() else 1.0


def update_subtitle_layer(self):
    """Function to update subtitle layer"""
    text = ''
    for subtitle in self.subtitles_list:
        if self.player_widget.position and (self.player_widget.position > subtitle[0] and self.player_widget.position < subtitle[0] + subtitle[1]):
            text = subtitle[2]
            break
    self.player_subtitle_layer.setSubtitleText(text)
    self.player_subtitle_layer.update()


def resize_player_widget(self, just_get_qrect=False):
    """Function to resize player widget (to accomodate video ratio inside screen space)"""
    # None
    if self.video_metadata:
        x = int(self.subtitles_panel_width_proportion * self.width())
        y = 20
        w = self.width() - x - 20
        h = self.height() - y - 20 - 200

        if w / h > self.video_metadata.get('width', 1920) / self.video_metadata.get('height', 1080):
            aspect_ratio = self.video_metadata.get('width', 1920) / self.video_metadata.get('height', 1080)
            new_h = h
            new_w = h * aspect_ratio
            qrect = QRect(int(x + (w - new_w) / 2), y, int(new_w), int(new_h))
        else:
            aspect_ratio = self.video_metadata.get('height', 1080) / self.video_metadata.get('width', 1920)
            new_h = w * aspect_ratio
            new_w = w
            qrect = QRect(x, int(y + ((h - new_h) / 2)), int(new_w), int(new_h))

        if just_get_qrect:
            return [int(qrect.x()), int(qrect.y()), int(qrect.width()), int(qrect.height())]
        else:
            self.player_border.setGeometry(qrect)

    # if self.video_metadata.get('width', 1920) > self.video_metadata.get('height', 1080):
    #     wp = 1
    #     hp = self.video_metadata.get('height', 1080) / self.video_metadata.get('width', 1920)
    # else:
    #     wp = self.video_metadata.get('width', 1920) / self.video_metadata.get('height', 1080)
    #     hp = 1

    # self.video_metadata.get('width', 1920) / self.video_metadata.get('height', 1080)
    # if self.video_metadata.get('width', 640) > self.video_metadata.get('height', 480):
    #    heigth_proportion = ((self.player_widget_area.width()*.7)-6) / self.video_metadata.get('width', 640)
    #    self.player_widget.setGeometry((self.width()*.2) + 3, (self.player_widget_area.height()*.5)-((heigth_proportion*self.video_metadata.get('height', 480))*.5), (self.player_widget_area.width()*.7)-6, self.video_metadata.get('height', 480)*heigth_proportion)
    # else:
    #    width_proportion = (self.player_widget_area.height()-7) / self.video_metadata.get('height', 480)
    #    self.player_widget.setGeometry((self.width()*.2) + ((self.player_widget_area.width()*.7)*.5)-((width_proportion*self.video_metadata.get('width', 640))*.5), 3, self.video_metadata.get('width', 640)*width_proportion, self.player_widget_area.height()-6)
    # self.player_border.setGeometry(self.player_widget.x()-3, self.player_widget.y()-3, self.player_widget.width()+6, self.player_widget.height()+6)
    # self.player_subtitle_layer.setGeometry(self.player_widget.x(), self.player_widget.y(), self.player_widget.width(), self.player_widget.height())
    # self.player_subtitle_textedit.setGeometry(self.player_widget.x()+(self.player_widget.width()*.1), self.player_widget.y()+(self.player_widget.height()*.5), self.player_widget.width()*.8, self.player_widget.height()*.4)


def update_controls(self):
    playercontrols.playercontrols_stop_button_clicked(self)


def update_timelines(self):
    self.timeline.update(self)
    if not self.subtitles_panel_findandreplace_panel.isVisible():
        subtitles_panel.update_subtitles_panel_widget_vision_content(self)



"""Module to handle subtitles

"""

from bisect import bisect
from subtitld.modules import history


def add_subtitle(subtitles=[], position=0.0, duration=5.0, text='', from_last_subtitle=False):
    """Function to add a subtitle to the main subtitle list"""
    history.history_append(subtitles)

    subt = [item[0] for item in subtitles]
    index = bisect(subt, position)

    if index:
        if subtitles[index - 1][0] + subtitles[index - 1][1] > position:
            subtitles[index - 1][1] -= (subtitles[index - 1][0] + subtitles[index - 1][1]) - position
        elif from_last_subtitle:
            position = (subtitles[index - 1][0] + subtitles[index - 1][1]) + .001

    if len(subtitles) - 1 > index and subtitles[index][0] - position < duration:
        duration = subtitles[index][0] - position

    subtitles.insert(index, [position, duration, text])

    return subtitles[subtitles.index([position, duration, text])]


def remove_subtitle(subtitles=[], selected_subtitle=False):
    """Function to add a subtitle to the main subtitle list"""
    if selected_subtitle:
        history.history_append(subtitles)

        subtitles.remove(selected_subtitle)
    return subtitles


def slice_subtitle(subtitles=[], selected_subtitle=False, position=0.0, last_text='', next_text='', strip_text=True):
    """Function to slice a subtitle in the main subtitle list"""
    if selected_subtitle and position > selected_subtitle[0] and position < (selected_subtitle[0] + selected_subtitle[1]):
        history.history_append(subtitles)

        index = subtitles.index(selected_subtitle)

        if position > subtitles[index][0] and position < (subtitles[index][0] + subtitles[index][1]):
            position_to_cut = position
        else:
            position_to_cut = (subtitles[index][0] + subtitles[index][1]) / 2

        new_duration = (subtitles[index][0] + subtitles[index][1]) - position_to_cut

        subtitles[index][1] = position_to_cut - subtitles[index][0] - 0.001
        subtitles[index][2] = last_text
        if strip_text:
            subtitles[index][2] = subtitles[index][2].strip()
            next_text = next_text.strip()

        return add_subtitle(subtitles=subtitles, position=position_to_cut, duration=new_duration, text=next_text)


def merge_back_subtitle(subtitles=[], selected_subtitle=False):
    """Function to merge a subtitle to the last in the main subtitle list"""
    if selected_subtitle and subtitles.index(selected_subtitle):
        history.history_append(subtitles)

        index = subtitles.index(selected_subtitle)
        subtitles[index - 1][1] = selected_subtitle[0] + selected_subtitle[1] - subtitles[index - 1][0]
        subtitles[index - 1][2] += ' ' + subtitles[index][2]

        remove_subtitle(subtitles=subtitles, selected_subtitle=subtitles[index])

        return subtitles[index - 1]


def merge_next_subtitle(subtitles=[], selected_subtitle=False):
    """Function to merge a subtitle to the next in the main subtitle list"""
    result = False
    if selected_subtitle and subtitles.index(selected_subtitle) < len(subtitles) - 1:
        history.history_append(subtitles)

        index = subtitles.index(selected_subtitle)
        subtitles[index][1] = subtitles[index + 1][0] + subtitles[index + 1][1] - selected_subtitle[0]
        subtitles[index][2] += ' ' + subtitles[index + 1][2]

        remove_subtitle(subtitles=subtitles, selected_subtitle=subtitles[index + 1])

        result = subtitles[index]
    return result


def move_subtitle(subtitles=[], selected_subtitle=False, amount=0.0):
    """Function to move a subtitle in the main subtitle list"""
    if selected_subtitle:
        history.history_append(subtitles)
        selected_subtitle[0] += amount


def move_start_subtitle(subtitles=[], selected_subtitle=False, amount=0.0, absolute_time=False, move_nereast=False):
    """Function to move the start a subtitle in the main subtitle list"""
    if selected_subtitle:
        history.history_append(subtitles)
        if move_nereast:
            if subtitles.index(selected_subtitle) - 1 >= 0 and round(subtitles[subtitles.index(selected_subtitle) - 1][0] + subtitles[subtitles.index(selected_subtitle) - 1][1], 3) == round(selected_subtitle[0] - .001, 3):
                if absolute_time:
                    subtitles[subtitles.index(selected_subtitle) - 1][1] = absolute_time - subtitles[subtitles.index(selected_subtitle) - 1][0] - 0.001
                else:
                    subtitles[subtitles.index(selected_subtitle) - 1][1] -= amount
        if absolute_time:
            selected_subtitle[1] = selected_subtitle[0] + selected_subtitle[1] - absolute_time
            selected_subtitle[0] = absolute_time
        else:
            selected_subtitle[0] += amount
            selected_subtitle[1] -= amount


def move_end_subtitle(subtitles=[], selected_subtitle=False, amount=0.0, absolute_time=False, move_nereast=False):
    """Function to move the end of a subtitle in the main subtitle list"""
    if selected_subtitle:
        history.history_append(subtitles)
        if move_nereast:
            if subtitles.index(selected_subtitle) + 1 <= len(subtitles) and round(subtitles[subtitles.index(selected_subtitle) + 1][0] - .001, 3) <= round(selected_subtitle[0] + selected_subtitle[1], 3):
                if absolute_time:
                    subtitles[subtitles.index(selected_subtitle) + 1][1] = subtitles[subtitles.index(selected_subtitle) + 1][0] + subtitles[subtitles.index(selected_subtitle) + 1][1] - absolute_time
                    subtitles[subtitles.index(selected_subtitle) + 1][0] = absolute_time
                else:
                    subtitles[subtitles.index(selected_subtitle) + 1][0] -= amount
                    subtitles[subtitles.index(selected_subtitle) + 1][1] -= amount
        if absolute_time:
            selected_subtitle[1] = absolute_time - selected_subtitle[0]
        else:
            selected_subtitle[1] += amount


def next_start_to_current_position(subtitles=[], position=0.0):
    """Function to set next start to position of a subtitle in the main subtitle list"""
    history.history_append(subtitles)
    subt = [item[0] for item in subtitles]
    index = bisect(subt, position)
    if index < len(subt):
        end = subtitles[index][0] + subtitles[index][1]
        subtitles[index][0] = position
        subtitles[index][1] = end - position
    if index and subtitles[index - 1][0] + subtitles[index - 1][1] > position:
        last_end_to_current_position(subtitles=subtitles, position=position - 0.001)


def subtitle_start_to_current_position(subtitles=[], position=0.0):
    """Function to set start to position of a subtitle in the main subtitle list"""
    history.history_append(subtitles)
    subt = [item[0] for item in subtitles]
    index = bisect(subt, position)
    if index and position < (subtitles[index - 1][0] + subtitles[index - 1][1]):
        end = subtitles[index - 1][0] + subtitles[index - 1][1]
        subtitles[index - 1][0] = position
        subtitles[index - 1][1] = end - position
    else:
        end = subtitles[index][0] + subtitles[index][1]
        subtitles[index][0] = position
        subtitles[index][1] = end - position


def subtitle_end_to_current_position(subtitles=[], position=0.0):
    """Function to set end to position of a subtitle in the main subtitle list"""
    history.history_append(subtitles)
    subt = [item[0] for item in subtitles]
    index = bisect(subt, position)
    if index:
        if position < (subtitles[index - 1][0] + subtitles[index - 1][1]):
            subtitles[index - 1][1] = position - subtitles[index - 1][0]
        else:
            subtitles[index - 1][1] = position - subtitles[index - 1][0]


def subtitle_under_current_position(subtitles=[], position=0.0):
    """Function to return subtitle under position"""
    current_subtitle = False
    subt = [item[0] for item in subtitles]
    index = bisect(subt, position)
    if index - 1 > -1 and position > subtitles[index - 1][0] and position < (subtitles[index - 1][0] + subtitles[index - 1][1]):
        current_subtitle = subtitles[index - 1]
    return current_subtitle, index - 1


def last_subtitle_current_position(subtitles=[], position=0.0):
    """Function to return the last subtitle of position"""
    last_subtitle = False
    subt = [item[0] for item in subtitles]
    index = bisect(subt, position)
    if index:
        if position > (subtitles[index - 1][0] + subtitles[index - 1][1]):
            last_subtitle = subtitles[index - 1]
    return last_subtitle


def next_subtitle_current_position(subtitles=[], position=0.0):
    """Function to return the next subtitle of position"""
    next_subtitle = False
    subt = [item[0] for item in subtitles]
    index = bisect(subt, position)
    if index < len(subt):
        next_subtitle = subtitles[index]
    return next_subtitle


def next_end_to_current_position(subtitles=[], position=0.0):
    """Function set next end to position"""
    history.history_append(subtitles)
    subt = [item[0] for item in subtitles]
    index = bisect(subt, position)
    if index:
        end = subtitles[index - 1][0] + subtitles[index - 1][1]
        if end > position:
            subtitles[index - 1][1] = position - subtitles[index - 1][0]


def last_end_to_current_position(subtitles=[], position=0.0):
    """Function set last end to position"""
    history.history_append(subtitles)
    subt = [item[0] for item in subtitles]
    index = bisect(subt, position)

    if index - 1 < len(subt) - 1:
        subtitles[index - 1][1] = position - subtitles[index - 1][0]


def last_start_to_current_position(subtitles=[], position=0.0):
    """Function set last start to position"""
    history.history_append(subtitles)
    subt = [item[0] for item in subtitles]
    index = bisect(subt, position)
    if index and subtitles[index - 1][0] < position and not (subtitles[index - 1][0] + subtitles[index - 1][1]) < position:
        end = subtitles[index - 1][0] + subtitles[index - 1][1]
        subtitles[index - 1][0] = position
        subtitles[index - 1][1] = end - position


def send_text_to_next_subtitle(subtitles=[], selected_subtitle=False, last_text='', next_text=''):
    """Function send text to the next subtitle"""
    if selected_subtitle and subtitles.index(selected_subtitle) + 1 < len(subtitles):
        history.history_append(subtitles)
        index = subtitles.index(selected_subtitle)
        subtitles[index][2] = last_text
        subtitles[index + 1][2] = next_text + ' ' + subtitles[index + 1][2]


def send_text_to_last_subtitle(subtitles=[], selected_subtitle=False, last_text='', next_text=''):
    """Function send text to the last subtitle"""
    if selected_subtitle and subtitles.index(selected_subtitle):
        history.history_append(subtitles)
        index = subtitles.index(selected_subtitle)
        subtitles[index][2] = next_text
        subtitles[index - 1][2] += ' ' + last_text


def set_gap(subtitles=[], position=0.0, gap=0.0):
    """Function to set gap in subtitles"""
    history.history_append(subtitles)
    for subtitle in subtitles:
        if subtitle[0] > position:
            subtitle[0] += gap


def change_subtitle_text(subtitles=[], selected_subtitle=False, text=''):
    """Function to change text of selected subtitle"""
    if selected_subtitle:
        history.history_append(subtitles)
        subtitles[subtitles.index(selected_subtitle)][2] = text


"""

Subtitld - open source subtitle editor

"""

import os
import sys
import datetime
import argparse

from PySide6.QtWidgets import QApplication, QWidget, QLabel, QGraphicsOpacityEffect, QMessageBox
from PySide6.QtGui import QIcon, QFont, QFontDatabase
from PySide6.QtCore import Qt, QRect, QPropertyAnimation, QTimer

from subtitld.modules import config
from subtitld.modules.history import history_redo, history_undo
from subtitld.modules.paths import PATH_SUBTITLD_DATA_THUMBNAILS, PATH_SUBTITLD_GRAPHICS, PATH_SUBTITLD_USER_CONFIG_FILE, ACTUAL_OS, LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS, LIST_OF_SUPPORTED_VIDEO_EXTENSIONS, PATH_SUBTITLD_DATA_BACKUP, get_graphics_path

if ACTUAL_OS == 'darwin':
    from subtitld.modules.paths import NSURL

list_of_supported_subtitle_extensions = []
for t in LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS:
    for ext in LIST_OF_SUPPORTED_SUBTITLE_EXTENSIONS[t]['extensions']:
        list_of_supported_subtitle_extensions.append(ext)
list_of_supported_subtitle_extensions = tuple(list_of_supported_subtitle_extensions)


parser = argparse.ArgumentParser(description='Subtitld is a software to create, edit and transcribe subtitles')
parser.add_argument('file', type=argparse.FileType('r'), help='The path for video or subtitle file', nargs='*', default=False)
parser.add_argument('--version', help='Prints the actual version of Subtitld.', action='store_true')
args = parser.parse_args()


class Subtitld(QWidget):
    """The main window (QWidget) class"""
    def __init__(self):
        super().__init__()
        self.setWindowTitle('Subtitld')
        self.setWindowIcon(QIcon(os.path.join(PATH_SUBTITLD_GRAPHICS, 'subtitld.png')))
        self.setAcceptDrops(True)

        self.setMinimumSize(860, 450)

        # Setting some default values
        self.update_accuracy = 200
        self.subtitles_list = []
        self.video_metadata = {}

        self.actual_subtitle_file = ''
        self.actual_video_file = ''

        self.settings = config.load(PATH_SUBTITLD_USER_CONFIG_FILE)

        self.selected_subtitle = False
        self.mediaplayer_zoom = 100.0
        self.video_waveformsize = .7
        self.mediaplayer_opacity = .5
        self.video_length = 0.01
        self.playback_speed = 1.0
        self.default_new_subtitle_duration = 5.0
        self.repeat_duration = 5.0
        self.repeat_duration_tmp = []
        self.repeat_times = 3
        self.repeat_activated = False
        self.unsaved = False
        self.format_to_save = 'SRT'
        self.format_usf_present = False
        self.selected_language = 'en'
        self.subtitles_panel_width_proportion = .3

        self.background_label = QLabel(self)
        self.background_label.setObjectName('background_label')

        self.start_screen_thumbnail_background = QLabel(parent=self)
        self.start_screen_thumbnail_background.setAlignment(Qt.AlignCenter)
        # self.start_screen_thumbnail_background.setScaledContents(True)
        self.start_screen_thumbnail_background_transparency = QGraphicsOpacityEffect()
        self.start_screen_thumbnail_background.setGraphicsEffect(self.start_screen_thumbnail_background_transparency)
        self.start_screen_thumbnail_background_transparency_animation = QPropertyAnimation(self.start_screen_thumbnail_background_transparency, b'opacity')
        # self.start_screen_thumbnail_background_transparency_animation.setEasingCurve(QEasingCurve.InExpo)
        self.start_screen_thumbnail_background_transparency.setOpacity(1)
        # self.start_screen_thumbnail_background_animation = QPropertyAnimation(self.start_screen_thumbnail_background, b'geometry')
        # self.start_screen_thumbnail_background_animation.setEasingCurve(QEasingCurve.OutCirc)

        self.background_label2 = QLabel(self)
        self.background_label2.setObjectName('background_label2')
        self.background_label2_transparency = QGraphicsOpacityEffect()
        self.background_label2.setGraphicsEffect(self.background_label2_transparency)
        self.background_label2_transparency_animation = QPropertyAnimation(self.background_label2_transparency, b'opacity')
        self.background_label2_transparency.setOpacity(1)

        # Setting the gorgeous watermarked background logo
        self.background_watermark_label = QLabel(self)
        self.background_watermark_label.setObjectName('background_watermark_label')

        # All the stylesheet properties are in a separate file, so importing it here
        self.setStyleSheet(open(os.path.join(PATH_SUBTITLD_GRAPHICS, 'stylesheet.qss')).read().replace('PATH_SUBTITLD_GRAPHICS/', get_graphics_path()))

        # The file io system
        from subtitld.modules import file_io
        self.file_io = file_io
        self.file_io.load(self)

        # if ACTUAL_OS == 'windows':
        #     # The windows update system
        #     from modules import update
        #     self.update = update
        #     self.update.load(self)

        # The start screen
        from subtitld.interface import startscreen
        self.startscreen = startscreen
        self.startscreen.load(self)
        self.startscreen.show(self)

        # The mpv video player{
        from subtitld.interface import player
        self.player = player
        self.player.load(self)

        from subtitld.interface import global_panel
        self.global_panel = global_panel
        self.global_panel.load(self)

        from subtitld.interface import subtitles_panel
        self.subtitles_panel = subtitles_panel
        self.subtitles_panel.load(self)

        from subtitld.interface import timeline
        self.timeline = timeline

        from subtitld.interface import playercontrols
        self.playercontrols = playercontrols
        self.playercontrols.load(self)

        # from subtitld.modules import playercontrols_properties
        # self.playercontrols_properties = playercontrols_properties
        # self.playercontrols_properties.load(self)

        # self.subtitles_panel.update_subtitles_panel_qlistwidget(self)
        # self.subtitles_panel.update_properties_widget(self)
        self.player.update(self)

        from subtitld.modules import shortcuts
        self.shortcuts = shortcuts
        self.shortcuts.load(self, self.settings['shortcuts'])

        self.autosave_timer = QTimer(self)
        self.autosave_timer.setInterval(int(self.settings['autosave'].get('interval', 300000)))
        self.autosave_timer.timeout.connect(lambda: autosave_timer_timeout(self))

        # Maybe implement saving window position...? Useful?
        # self.setGeometry(0, 0, QDesktopWidget().screenGeometry().width(), QDesktopWidget().screenGeometry().height())
        self.showMaximized()
        # self.setFixedSize(QSize(1280, 720))

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls and len(event.mimeData().urls()) > 0:
            if sys.platform == 'darwin':
                filename = str(NSURL.alloc().initWithString_(event.mimeData().urls()[0].toString()).fileSystemRepresentation())
            else:
                filename = event.mimeData().urls()[0].toLocalFile()

            if filename.lower().endswith(('.subtitld')) or filename.lower().endswith(list_of_supported_subtitle_extensions) or filename.lower().endswith(LIST_OF_SUPPORTED_VIDEO_EXTENSIONS):
                event.accept()

    def dropEvent(self, event):
        event.accept()
        # if sys.platform == 'darwin':
        #     filename = str(NSURL.alloc().initWithString_(event.mimeData().urls()[0].toString()).fileSystemRepresentation())
        # else:
        #     filename = event.mimeData().urls()[0].toLocalFile()

    def resizeEvent(self, event):
        self.background_label.setGeometry(0, 0, self.width(), self.height())
        self.start_screen_thumbnail_background.setGeometry(0, 0, self.width(), self.height())
        self.background_label2.setGeometry(0, 0, self.width(), self.height())

        self.startscreen.resized(self)
        self.subtitles_panel.resized(self)
        # self.properties.resized(self)

        self.global_panel.resized(self)
        # self.global_properties_panel.resized(self)

        self.playercontrols.resized(self)
        # self.playercontrols_properties.resized(self)

        self.background_watermark_label.setGeometry(int((self.width() * .5) - 129), int(((self.height() - self.playercontrols_widget.height()) * .5) - 129), 258, 258)
        # self.background_watermark_label.setVisible(False)

        self.player.resized(self)
        self.timeline.resized(self)
        event.accept()

    def closeEvent(self, event):
        if self.unsaved:
            save_message_box = QMessageBox(self)

            save_message_box.setWindowTitle(self.tr('Unsaved changes'))
            save_message_box.setText(
                self.tr('Do you want to save the changes you made on the subtitles?')
            )
            save_message_box.addButton(self.tr('Save'), QMessageBox.AcceptRole)
            save_message_box.addButton(self.tr("Don't save"), QMessageBox.RejectRole)
            ret = save_message_box.exec_()

            if ret == QMessageBox.AcceptRole:
                self.subtitles_panel.toppanel_save_button_clicked(self)

        self.thread_get_waveform.quit()
        self.thread_get_qimages.quit()
        # self.thread_extract_scene_time_positions.quit()
        self.thread_generated_burned_video.quit()
        self.thread_extract_waveform.quit()
        if self.actual_subtitle_file and 'hash' in self.video_metadata:
            self.player_widget.grab().save(os.path.join(PATH_SUBTITLD_DATA_THUMBNAILS, self.video_metadata['hash'] + '.png'))
            self.settings['recent_files'][self.actual_subtitle_file]['last_position'] = self.player_widget.position
        config.save(self.settings, PATH_SUBTITLD_USER_CONFIG_FILE)
        self.player_widget.close()

        event.accept()

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Space:
            self.player_widget.pause()
            self.playercontrols_playpause_button.setChecked(not self.playercontrols_playpause_button.isChecked())
            self.playercontrols.playercontrols_playpause_button_update(self)

        if event.key() == Qt.Key_F1:
            self.playercontrols.add_subtitle_button_clicked(self)

        if event.key() == Qt.Key_F12:
            self.playercontrols.add_subtitle_button_clicked(self)

        if event.key() == Qt.Key_Z:
            if event.modifiers() == Qt.ControlModifier | Qt.ShiftModifier:
                history_redo(actual_subtitles=self.subtitles_list)
            elif event.modifiers() == Qt.ControlModifier:
                history_undo(actual_subtitles=self.subtitles_list)
            self.selected_subtitle = False
            self.subtitles_panel.update_subtitles_panel_qlistwidget(self)
            # self.properties.update_properties_widget(self)
            self.timeline.update(self)

        if event.key() == Qt.Key_Left:
            self.player_widget.frameBackStep()

        if event.key() == Qt.Key_Right:
            self.player_widget.frameStep()

    def generate_effect(self, widget, effect_type, duration, startValue, endValue):
        widget.setDuration(duration)
        if effect_type == 'geometry':
            widget.setStartValue(QRect(startValue[0], startValue[1], startValue[2], startValue[3]))
            widget.setEndValue(QRect(endValue[0], endValue[1], endValue[2], endValue[3]))
        if effect_type in ['maximumHeight', 'maximumWidth']:
            widget.setStartValue(startValue)
            widget.setEndValue(endValue)
        elif effect_type == 'opacity':
            widget.setStartValue(startValue)
            widget.setEndValue(endValue)
        widget.start()


def autosave_timer_timeout(self):
    filename = os.path.basename(self.actual_subtitle_file).rsplit('.', 1)[0]
    if not filename:
        filename = os.path.basename(self.video_metadata['filepath']).rsplit('.', 1)[0]
    self.file_io.save_file(os.path.join(PATH_SUBTITLD_DATA_BACKUP, filename + '_' + datetime.datetime.now().strftime("%Y%m%d%H%M%S") + '.srt'), self.subtitles_list, 'SRT')


def main():
    app = QApplication(sys.argv)
    # command to update ts files: pylupdate5 subtitld.py modules/*.py -ts locale/en_US.ts
    # if os.path.isfile(os.path.join(PATH_LOCALE, 'en_US.qm')):
    #     translator = QTranslator()
    #     translator.load(os.path.join(PATH_LOCALE, 'en_US.qm'))
    #     app.installTranslator(translator)
    # app.setDesktopSettingsAware(False)
    # app.setStyle("plastique")
    app.setApplicationName("Subtitld")

    font_database = QFontDatabase()
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'Ubuntu-B.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'Ubuntu-BI.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'Ubuntu-C.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'Ubuntu-L.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'Ubuntu-LI.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'Ubuntu-M.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'Ubuntu-MI.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'UbuntuMono-B.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'UbuntuMono-BI.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'UbuntuMono-R.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'UbuntuMono-RI.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'Ubuntu-R.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'Ubuntu-RI.ttf'))
    font_database.addApplicationFont(os.path.join(PATH_SUBTITLD_GRAPHICS, 'Ubuntu-Th.ttf'))

    app.setFont(QFont('Ubuntu', 10))
    app.main = Subtitld()
    app.main.show()

    if args.file:
        files_list = []
        for filepath in args.file:
            files_list.append(filepath.name)
            app.main.file_io.open_filepath(app.main, files_to_open=files_list)

    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



"""Subtitles Video panel

"""

from PySide6.QtWidgets import QPushButton, QWidget, QVBoxLayout, QTabWidget, QLabel, QHBoxLayout, QSpinBox, QComboBox, QGroupBox, QColorDialog
from PySide6.QtCore import Qt
from PySide6.QtGui import QFontDatabase, QColor

from subtitld.interface import global_panel


def load_menu(self):
    """Function to load subtitles panel widgets"""
    self.global_panel_interface_menu_button = QPushButton('Interface')
    self.global_panel_interface_menu_button.setCheckable(True)
    self.global_panel_interface_menu_button.setProperty('class', 'global_panel_menu')
    self.global_panel_interface_menu_button.clicked.connect(lambda: global_panel_menu_changed(self))
    self.global_panel_menu.layout().addWidget(self.global_panel_interface_menu_button)


def global_panel_menu_changed(self):
    self.global_panel_interface_menu_button.setEnabled(False)
    global_panel.global_panel_menu_changed(self, self.global_panel_interface_menu_button, self.global_panel_interface_content)


def load_widgets(self):
    """Function to load subtitles panel widgets"""

    self.global_panel_interface_content = QWidget()
    self.global_panel_interface_content.setLayout(QVBoxLayout())
    self.global_panel_interface_content.layout().setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_content.layout().setSpacing(20)

    self.global_panel_interface_tabwidget = QTabWidget()

    self.global_panel_interface_tabwidget_videoplayer = QWidget()
    self.global_panel_interface_tabwidget_videoplayer.setLayout(QVBoxLayout())
    self.global_panel_interface_tabwidget_videoplayer.layout().setContentsMargins(10, 10, 10, 10)
    self.global_panel_interface_tabwidget_videoplayer.layout().setSpacing(20)

    self.global_panel_interface_videoplayer_font_group = QGroupBox('Font')
    self.global_panel_interface_videoplayer_font_group.setLayout(QHBoxLayout())
    self.global_panel_interface_videoplayer_font_group.layout().setContentsMargins(10, 10, 10, 10)
    self.global_panel_interface_videoplayer_font_group.layout().setSpacing(20)

    self.global_panel_interface_videoplayer_fontsize_line = QVBoxLayout()
    self.global_panel_interface_videoplayer_fontsize_line.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_fontsize_line.setSpacing(2)

    self.global_panel_interface_videoplayer_fontsize_label = QLabel('Size')
    self.global_panel_interface_videoplayer_fontsize_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_fontsize_line.addWidget(self.global_panel_interface_videoplayer_fontsize_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_fontsize_line_2 = QHBoxLayout()
    self.global_panel_interface_videoplayer_fontsize_line_2.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_fontsize_line_2.setSpacing(5)

    self.global_panel_interface_videoplayer_fontsize_spinbox = QSpinBox()
    self.global_panel_interface_videoplayer_fontsize_spinbox.setMinimum(1)
    self.global_panel_interface_videoplayer_fontsize_spinbox.setMaximum(999)
    self.global_panel_interface_videoplayer_fontsize_spinbox.valueChanged.connect(lambda: global_panel_interface_videoplayer_fontsize_spinbox_changed(self))
    self.global_panel_interface_videoplayer_fontsize_line_2.addWidget(self.global_panel_interface_videoplayer_fontsize_spinbox, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_fontsize_seconds_label = QLabel('Pixels')
    self.global_panel_interface_videoplayer_fontsize_seconds_label.setProperty('class', 'units_label')
    self.global_panel_interface_videoplayer_fontsize_line_2.addWidget(self.global_panel_interface_videoplayer_fontsize_seconds_label, 0, Qt.AlignLeft)
    # self.global_panel_interface_videoplayer_fontsize_line_2.addStretch()

    self.global_panel_interface_videoplayer_fontsize_line.addLayout(self.global_panel_interface_videoplayer_fontsize_line_2)

    self.global_panel_interface_videoplayer_font_group.layout().addLayout(self.global_panel_interface_videoplayer_fontsize_line)

    self.global_panel_interface_videoplayer_fontfamily_line = QVBoxLayout()
    self.global_panel_interface_videoplayer_fontfamily_line.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_fontfamily_line.setSpacing(2)

    self.global_panel_interface_videoplayer_fontfamily_label = QLabel('Family')
    self.global_panel_interface_videoplayer_fontfamily_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_fontfamily_line.addWidget(self.global_panel_interface_videoplayer_fontfamily_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_fontfamily_line_2 = QHBoxLayout()
    self.global_panel_interface_videoplayer_fontfamily_line_2.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_fontfamily_line_2.setSpacing(5)

    fonts = QFontDatabase().families()
    self.global_panel_interface_videoplayer_fontfamily_combobox = QComboBox()
    self.global_panel_interface_videoplayer_fontfamily_combobox.addItems(fonts)
    self.global_panel_interface_videoplayer_fontfamily_combobox.activated.connect(lambda: global_panel_interface_videoplayer_fontfamily_combobox_changed(self))
    self.global_panel_interface_videoplayer_fontfamily_line_2.addWidget(self.global_panel_interface_videoplayer_fontfamily_combobox, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_fontfamily_line.addLayout(self.global_panel_interface_videoplayer_fontfamily_line_2)

    self.global_panel_interface_videoplayer_font_group.layout().addLayout(self.global_panel_interface_videoplayer_fontfamily_line)

    self.global_panel_interface_videoplayer_color_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_color_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_color_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_color_label = QLabel('Color')
    self.global_panel_interface_videoplayer_color_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_color_vbox.addWidget(self.global_panel_interface_videoplayer_color_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_color_button = QPushButton()
    self.global_panel_interface_videoplayer_color_button.setProperty('class', 'color_pick_button')
    self.global_panel_interface_videoplayer_color_button.setFixedWidth(80)
    self.global_panel_interface_videoplayer_color_button.clicked.connect(lambda: global_panel_interface_videoplayer_color_button_clicked(self))

    self.global_panel_interface_videoplayer_color_vbox.addWidget(self.global_panel_interface_videoplayer_color_button, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_font_group.layout().addLayout(self.global_panel_interface_videoplayer_color_vbox)

    self.global_panel_interface_videoplayer_font_group.layout().addStretch()

    self.global_panel_interface_tabwidget_videoplayer.layout().addWidget(self.global_panel_interface_videoplayer_font_group)

    self.global_panel_interface_videoplayer_shadow_group = QGroupBox('Shadow')
    self.global_panel_interface_videoplayer_shadow_group.setCheckable(True)
    self.global_panel_interface_videoplayer_shadow_group.setLayout(QHBoxLayout())
    self.global_panel_interface_videoplayer_shadow_group.toggled.connect(lambda: global_panel_interface_videoplayer_shadow_group_toggled(self))
    self.global_panel_interface_videoplayer_shadow_group.layout().setContentsMargins(10, 10, 10, 10)
    self.global_panel_interface_videoplayer_shadow_group.layout().setSpacing(20)

    self.global_panel_interface_videoplayer_shadow_x_position_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_shadow_x_position_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_shadow_x_position_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_shadow_x_position_label = QLabel('X position')
    self.global_panel_interface_videoplayer_shadow_x_position_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_shadow_x_position_vbox.addWidget(self.global_panel_interface_videoplayer_shadow_x_position_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_shadow_x_position_line = QHBoxLayout()
    self.global_panel_interface_videoplayer_shadow_x_position_line.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_shadow_x_position_line.setSpacing(5)

    self.global_panel_interface_videoplayer_shadow_x_position = QSpinBox()
    self.global_panel_interface_videoplayer_shadow_x_position.setMinimum(-99999)
    self.global_panel_interface_videoplayer_shadow_x_position.setMaximum(99999)
    self.global_panel_interface_videoplayer_shadow_x_position.valueChanged.connect(lambda: global_panel_interface_videoplayer_shadow_x_position_changed(self))
    self.global_panel_interface_videoplayer_shadow_x_position_line.addWidget(self.global_panel_interface_videoplayer_shadow_x_position, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_shadow_x_position_pixels_label = QLabel('Pixels')
    self.global_panel_interface_videoplayer_shadow_x_position_pixels_label.setProperty('class', 'units_label')
    self.global_panel_interface_videoplayer_shadow_x_position_line.addWidget(self.global_panel_interface_videoplayer_shadow_x_position_pixels_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_shadow_x_position_vbox.addLayout(self.global_panel_interface_videoplayer_shadow_x_position_line)

    self.global_panel_interface_videoplayer_shadow_group.layout().addLayout(self.global_panel_interface_videoplayer_shadow_x_position_vbox)

    self.global_panel_interface_videoplayer_shadow_y_position_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_shadow_y_position_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_shadow_y_position_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_shadow_y_position_label = QLabel('Y position')
    self.global_panel_interface_videoplayer_shadow_y_position_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_shadow_y_position_vbox.addWidget(self.global_panel_interface_videoplayer_shadow_y_position_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_shadow_y_position_line = QHBoxLayout()
    self.global_panel_interface_videoplayer_shadow_y_position_line.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_shadow_y_position_line.setSpacing(5)

    self.global_panel_interface_videoplayer_shadow_y_position = QSpinBox()
    self.global_panel_interface_videoplayer_shadow_y_position.setMinimum(-99999)
    self.global_panel_interface_videoplayer_shadow_y_position.setMaximum(99999)
    self.global_panel_interface_videoplayer_shadow_y_position.valueChanged.connect(lambda: global_panel_interface_videoplayer_shadow_y_position_changed(self))
    self.global_panel_interface_videoplayer_shadow_y_position_line.addWidget(self.global_panel_interface_videoplayer_shadow_y_position, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_shadow_y_position_pixels_label = QLabel('Pixels')
    self.global_panel_interface_videoplayer_shadow_y_position_pixels_label.setProperty('class', 'units_label')
    self.global_panel_interface_videoplayer_shadow_y_position_line.addWidget(self.global_panel_interface_videoplayer_shadow_y_position_pixels_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_shadow_y_position_vbox.addLayout(self.global_panel_interface_videoplayer_shadow_y_position_line)

    self.global_panel_interface_videoplayer_shadow_group.layout().addLayout(self.global_panel_interface_videoplayer_shadow_y_position_vbox)

    self.global_panel_interface_tabwidget_videoplayer.layout().addWidget(self.global_panel_interface_videoplayer_shadow_group)

    self.global_panel_interface_videoplayer_shadow_color_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_shadow_color_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_shadow_color_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_shadow_color_label = QLabel('Color')
    self.global_panel_interface_videoplayer_shadow_color_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_shadow_color_vbox.addWidget(self.global_panel_interface_videoplayer_shadow_color_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_shadow_color_button = QPushButton()
    self.global_panel_interface_videoplayer_shadow_color_button.setProperty('class', 'color_pick_button')
    self.global_panel_interface_videoplayer_shadow_color_button.setFixedWidth(80)
    self.global_panel_interface_videoplayer_shadow_color_button.clicked.connect(lambda: global_panel_interface_videoplayer_shadow_color_button_clicked(self))

    self.global_panel_interface_videoplayer_shadow_color_vbox.addWidget(self.global_panel_interface_videoplayer_shadow_color_button, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_shadow_group.layout().addLayout(self.global_panel_interface_videoplayer_shadow_color_vbox)

    self.global_panel_interface_videoplayer_shadow_group.layout().addStretch()

    self.global_panel_interface_tabwidget_videoplayer.layout().addWidget(self.global_panel_interface_videoplayer_shadow_group)

    self.global_panel_interface_videoplayer_backgroundbox_group = QGroupBox('Background box')
    self.global_panel_interface_videoplayer_backgroundbox_group.setCheckable(True)
    self.global_panel_interface_videoplayer_backgroundbox_group.setLayout(QHBoxLayout())
    self.global_panel_interface_videoplayer_backgroundbox_group.toggled.connect(lambda: global_panel_interface_videoplayer_backgroundbox_group_toggled(self))
    self.global_panel_interface_videoplayer_backgroundbox_group.layout().setContentsMargins(10, 10, 10, 10)
    self.global_panel_interface_videoplayer_backgroundbox_group.layout().setSpacing(20)

    self.global_panel_interface_videoplayer_backgroundbox_padding_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_backgroundbox_padding_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_backgroundbox_padding_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_backgroundbox_padding_label = QLabel('Padding')
    self.global_panel_interface_videoplayer_backgroundbox_padding_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_backgroundbox_padding_vbox.addWidget(self.global_panel_interface_videoplayer_backgroundbox_padding_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_backgroundbox_padding_line = QHBoxLayout()
    self.global_panel_interface_videoplayer_backgroundbox_padding_line.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_backgroundbox_padding_line.setSpacing(5)

    self.global_panel_interface_videoplayer_backgroundbox_padding = QSpinBox()
    self.global_panel_interface_videoplayer_backgroundbox_padding.setMinimum(-99999)
    self.global_panel_interface_videoplayer_backgroundbox_padding.setMaximum(99999)
    self.global_panel_interface_videoplayer_backgroundbox_padding.valueChanged.connect(lambda: global_panel_interface_videoplayer_backgroundbox_padding_changed(self))
    self.global_panel_interface_videoplayer_backgroundbox_padding_line.addWidget(self.global_panel_interface_videoplayer_backgroundbox_padding, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_backgroundbox_padding_pixels_label = QLabel('Pixels')
    self.global_panel_interface_videoplayer_backgroundbox_padding_pixels_label.setProperty('class', 'units_label')
    self.global_panel_interface_videoplayer_backgroundbox_padding_line.addWidget(self.global_panel_interface_videoplayer_backgroundbox_padding_pixels_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_backgroundbox_padding_vbox.addLayout(self.global_panel_interface_videoplayer_backgroundbox_padding_line)

    self.global_panel_interface_videoplayer_backgroundbox_group.layout().addLayout(self.global_panel_interface_videoplayer_backgroundbox_padding_vbox)

    self.global_panel_interface_videoplayer_backgroundbox_border_radius_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_backgroundbox_border_radius_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_backgroundbox_border_radius_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_backgroundbox_border_radius_label = QLabel('Border radius')
    self.global_panel_interface_videoplayer_backgroundbox_border_radius_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_backgroundbox_border_radius_vbox.addWidget(self.global_panel_interface_videoplayer_backgroundbox_border_radius_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_backgroundbox_border_radius_line = QHBoxLayout()
    self.global_panel_interface_videoplayer_backgroundbox_border_radius_line.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_backgroundbox_border_radius_line.setSpacing(5)

    self.global_panel_interface_videoplayer_backgroundbox_border_radius = QSpinBox()
    self.global_panel_interface_videoplayer_backgroundbox_border_radius.setMinimum(0)
    self.global_panel_interface_videoplayer_backgroundbox_border_radius.setMaximum(99999)
    self.global_panel_interface_videoplayer_backgroundbox_border_radius.valueChanged.connect(lambda: global_panel_interface_videoplayer_backgroundbox_border_radius_changed(self))
    self.global_panel_interface_videoplayer_backgroundbox_border_radius_line.addWidget(self.global_panel_interface_videoplayer_backgroundbox_border_radius, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_backgroundbox_border_radius_pixels_label = QLabel('%')
    self.global_panel_interface_videoplayer_backgroundbox_border_radius_pixels_label.setProperty('class', 'units_label')
    self.global_panel_interface_videoplayer_backgroundbox_border_radius_line.addWidget(self.global_panel_interface_videoplayer_backgroundbox_border_radius_pixels_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_backgroundbox_border_radius_vbox.addLayout(self.global_panel_interface_videoplayer_backgroundbox_border_radius_line)

    self.global_panel_interface_videoplayer_backgroundbox_group.layout().addLayout(self.global_panel_interface_videoplayer_backgroundbox_border_radius_vbox)

    self.global_panel_interface_videoplayer_backgroundbox_color_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_backgroundbox_color_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_backgroundbox_color_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_backgroundbox_color_label = QLabel('Color')
    self.global_panel_interface_videoplayer_backgroundbox_color_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_backgroundbox_color_vbox.addWidget(self.global_panel_interface_videoplayer_backgroundbox_color_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_backgroundbox_color_button = QPushButton()
    self.global_panel_interface_videoplayer_backgroundbox_color_button.setProperty('class', 'color_pick_button')
    self.global_panel_interface_videoplayer_backgroundbox_color_button.setFixedWidth(80)
    self.global_panel_interface_videoplayer_backgroundbox_color_button.clicked.connect(lambda: global_panel_interface_videoplayer_backgroundbox_color_button_clicked(self))

    self.global_panel_interface_videoplayer_backgroundbox_color_vbox.addWidget(self.global_panel_interface_videoplayer_backgroundbox_color_button, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_backgroundbox_group.layout().addLayout(self.global_panel_interface_videoplayer_backgroundbox_color_vbox)

    self.global_panel_interface_videoplayer_backgroundbox_group.layout().addStretch()

    self.global_panel_interface_tabwidget_videoplayer.layout().addWidget(self.global_panel_interface_videoplayer_backgroundbox_group)

    self.global_panel_interface_videoplayer_safe_margin_action_group = QGroupBox('Action safe margin')
    self.global_panel_interface_videoplayer_safe_margin_action_group.setCheckable(True)
    self.global_panel_interface_videoplayer_safe_margin_action_group.setLayout(QHBoxLayout())
    self.global_panel_interface_videoplayer_safe_margin_action_group.toggled.connect(lambda: global_panel_interface_videoplayer_safe_margin_action_group_toggled(self))
    self.global_panel_interface_videoplayer_safe_margin_action_group.layout().setContentsMargins(10, 10, 10, 10)
    self.global_panel_interface_videoplayer_safe_margin_action_group.layout().setSpacing(20)

    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_label = QLabel('Horizontal margin')
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_vbox.addWidget(self.global_panel_interface_videoplayer_safe_margin_action_x_margin_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_line = QHBoxLayout()
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_line.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_line.setSpacing(5)

    self.global_panel_interface_videoplayer_safe_margin_action_x_margin = QSpinBox()
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin.setMinimum(0)
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin.setMaximum(50)
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin.valueChanged.connect(lambda: global_panel_interface_videoplayer_safe_margin_action_x_margin_changed(self))
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_line.addWidget(self.global_panel_interface_videoplayer_safe_margin_action_x_margin, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_pixels_label = QLabel('%')
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_pixels_label.setProperty('class', 'units_label')
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_line.addWidget(self.global_panel_interface_videoplayer_safe_margin_action_x_margin_pixels_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_action_x_margin_vbox.addLayout(self.global_panel_interface_videoplayer_safe_margin_action_x_margin_line)

    self.global_panel_interface_videoplayer_safe_margin_action_group.layout().addLayout(self.global_panel_interface_videoplayer_safe_margin_action_x_margin_vbox)

    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_label = QLabel('Vertical margin')
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_vbox.addWidget(self.global_panel_interface_videoplayer_safe_margin_action_y_margin_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_line = QHBoxLayout()
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_line.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_line.setSpacing(5)

    self.global_panel_interface_videoplayer_safe_margin_action_y_margin = QSpinBox()
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin.setMinimum(0)
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin.setMaximum(50)
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin.valueChanged.connect(lambda: global_panel_interface_videoplayer_safe_margin_action_y_margin_changed(self))
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_line.addWidget(self.global_panel_interface_videoplayer_safe_margin_action_y_margin, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_pixels_label = QLabel('Pixels')
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_pixels_label.setProperty('class', 'units_label')
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_line.addWidget(self.global_panel_interface_videoplayer_safe_margin_action_y_margin_pixels_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_action_y_margin_vbox.addLayout(self.global_panel_interface_videoplayer_safe_margin_action_y_margin_line)

    self.global_panel_interface_videoplayer_safe_margin_action_group.layout().addLayout(self.global_panel_interface_videoplayer_safe_margin_action_y_margin_vbox)

    self.global_panel_interface_videoplayer_safe_margin_action_color_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_safe_margin_action_color_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_safe_margin_action_color_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_safe_margin_action_color_label = QLabel('Color')
    self.global_panel_interface_videoplayer_safe_margin_action_color_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_safe_margin_action_color_vbox.addWidget(self.global_panel_interface_videoplayer_safe_margin_action_color_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_action_color_button = QPushButton()
    self.global_panel_interface_videoplayer_safe_margin_action_color_button.setProperty('class', 'color_pick_button')
    self.global_panel_interface_videoplayer_safe_margin_action_color_button.setFixedWidth(80)
    self.global_panel_interface_videoplayer_safe_margin_action_color_button.clicked.connect(lambda: global_panel_interface_videoplayer_safe_margin_action_color_button_clicked(self))

    self.global_panel_interface_videoplayer_safe_margin_action_color_vbox.addWidget(self.global_panel_interface_videoplayer_safe_margin_action_color_button, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_action_group.layout().addLayout(self.global_panel_interface_videoplayer_safe_margin_action_color_vbox)

    self.global_panel_interface_videoplayer_safe_margin_action_group.layout().addStretch()

    self.global_panel_interface_tabwidget_videoplayer.layout().addWidget(self.global_panel_interface_videoplayer_safe_margin_action_group)

    self.global_panel_interface_videoplayer_safe_margin_title_group = QGroupBox('Title safe margin')
    self.global_panel_interface_videoplayer_safe_margin_title_group.setCheckable(True)
    self.global_panel_interface_videoplayer_safe_margin_title_group.setLayout(QHBoxLayout())
    self.global_panel_interface_videoplayer_safe_margin_title_group.toggled.connect(lambda: global_panel_interface_videoplayer_safe_margin_title_group_toggled(self))
    self.global_panel_interface_videoplayer_safe_margin_title_group.layout().setContentsMargins(10, 10, 10, 10)
    self.global_panel_interface_videoplayer_safe_margin_title_group.layout().setSpacing(20)

    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_label = QLabel('Horizontal margin')
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_vbox.addWidget(self.global_panel_interface_videoplayer_safe_margin_title_x_margin_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_line = QHBoxLayout()
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_line.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_line.setSpacing(5)

    self.global_panel_interface_videoplayer_safe_margin_title_x_margin = QSpinBox()
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin.setMinimum(0)
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin.setMaximum(50)
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin.valueChanged.connect(lambda: global_panel_interface_videoplayer_safe_margin_title_x_margin_changed(self))
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_line.addWidget(self.global_panel_interface_videoplayer_safe_margin_title_x_margin, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_pixels_label = QLabel('Pixels')
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_pixels_label.setProperty('class', 'units_label')
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_line.addWidget(self.global_panel_interface_videoplayer_safe_margin_title_x_margin_pixels_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_title_x_margin_vbox.addLayout(self.global_panel_interface_videoplayer_safe_margin_title_x_margin_line)

    self.global_panel_interface_videoplayer_safe_margin_title_group.layout().addLayout(self.global_panel_interface_videoplayer_safe_margin_title_x_margin_vbox)

    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_label = QLabel('Vertical margin')
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_vbox.addWidget(self.global_panel_interface_videoplayer_safe_margin_title_y_margin_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_line = QHBoxLayout()
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_line.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_line.setSpacing(5)

    self.global_panel_interface_videoplayer_safe_margin_title_y_margin = QSpinBox()
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin.setMinimum(0)
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin.setMaximum(50)
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin.valueChanged.connect(lambda: global_panel_interface_videoplayer_safe_margin_title_y_margin_changed(self))
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_line.addWidget(self.global_panel_interface_videoplayer_safe_margin_title_y_margin, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_pixels_label = QLabel('Pixels')
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_pixels_label.setProperty('class', 'units_label')
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_line.addWidget(self.global_panel_interface_videoplayer_safe_margin_title_y_margin_pixels_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_title_y_margin_vbox.addLayout(self.global_panel_interface_videoplayer_safe_margin_title_y_margin_line)

    self.global_panel_interface_videoplayer_safe_margin_title_group.layout().addLayout(self.global_panel_interface_videoplayer_safe_margin_title_y_margin_vbox)

    self.global_panel_interface_videoplayer_safe_margin_title_color_vbox = QVBoxLayout()
    self.global_panel_interface_videoplayer_safe_margin_title_color_vbox.setContentsMargins(0, 0, 0, 0)
    self.global_panel_interface_videoplayer_safe_margin_title_color_vbox.setSpacing(2)

    self.global_panel_interface_videoplayer_safe_margin_title_color_label = QLabel('Color')
    self.global_panel_interface_videoplayer_safe_margin_title_color_label.setProperty('class', 'widget_label')
    self.global_panel_interface_videoplayer_safe_margin_title_color_vbox.addWidget(self.global_panel_interface_videoplayer_safe_margin_title_color_label, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_title_color_button = QPushButton()
    self.global_panel_interface_videoplayer_safe_margin_title_color_button.setProperty('class', 'color_pick_button')
    self.global_panel_interface_videoplayer_safe_margin_title_color_button.setFixedWidth(80)
    self.global_panel_interface_videoplayer_safe_margin_title_color_button.clicked.connect(lambda: global_panel_interface_videoplayer_safe_margin_title_color_button_clicked(self))

    self.global_panel_interface_videoplayer_safe_margin_title_color_vbox.addWidget(self.global_panel_interface_videoplayer_safe_margin_title_color_button, 0, Qt.AlignLeft)

    self.global_panel_interface_videoplayer_safe_margin_title_group.layout().addLayout(self.global_panel_interface_videoplayer_safe_margin_title_color_vbox)

    self.global_panel_interface_videoplayer_safe_margin_title_group.layout().addStretch()

    self.global_panel_interface_tabwidget_videoplayer.layout().addWidget(self.global_panel_interface_videoplayer_safe_margin_title_group)

    self.global_panel_interface_tabwidget_videoplayer.layout().addStretch()

    self.global_panel_interface_tabwidget.addTab(self.global_panel_interface_tabwidget_videoplayer, 'Video Player')

    self.global_panel_interface_content.layout().addWidget(self.global_panel_interface_tabwidget)

    self.global_panel_content_stacked_widgets.addWidget(self.global_panel_interface_content)

    update_widgets(self)


def global_panel_interface_videoplayer_fontfamily_combobox_changed(self):
    self.settings['videoplayer']['font_family'] = self.global_panel_interface_videoplayer_fontfamily_combobox.currentText()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_fontsize_spinbox_changed(self):
    self.settings['videoplayer']['font_size'] = self.global_panel_interface_videoplayer_fontsize_spinbox.value()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_shadow_group_toggled(self):
    self.settings['videoplayer']['shadow_enabled'] = self.global_panel_interface_videoplayer_shadow_group.isChecked()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_shadow_x_position_changed(self):
    self.settings['videoplayer']['shadow_x'] = self.global_panel_interface_videoplayer_shadow_x_position.value()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_shadow_y_position_changed(self):
    self.settings['videoplayer']['shadow_y'] = self.global_panel_interface_videoplayer_shadow_y_position.value()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_shadow_color_button_clicked(self):
    color = QColorDialog.getColor(options=QColorDialog.ShowAlphaChannel)
    if color.isValid():
        self.settings['videoplayer']['shadow_color'] = color.name(QColor.HexArgb)
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])
    update_widgets(self)


def global_panel_interface_videoplayer_color_button_clicked(self):
    color = QColorDialog.getColor(options=QColorDialog.ShowAlphaChannel)
    if color.isValid():
        self.settings['videoplayer']['color'] = color.name(QColor.HexArgb)
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])
    update_widgets(self)


def global_panel_interface_videoplayer_backgroundbox_group_toggled(self):
    self.settings['videoplayer']['safe_margin_action_enabled'] = self.global_panel_interface_videoplayer_backgroundbox_group.isChecked()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_backgroundbox_padding_changed(self):
    self.settings['videoplayer']['backgroundbox_padding'] = self.global_panel_interface_videoplayer_backgroundbox_padding.value()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_backgroundbox_border_radius_changed(self):
    self.settings['videoplayer']['backgroundbox_border_radius'] = self.global_panel_interface_videoplayer_backgroundbox_border_radius.value()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_backgroundbox_color_button_clicked(self):
    color = QColorDialog.getColor(options=QColorDialog.ShowAlphaChannel)
    if color.isValid():
        self.settings['videoplayer']['backgroundbox_color'] = color.name(QColor.HexArgb)
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])
    update_widgets(self)


def global_panel_interface_videoplayer_safe_margin_action_group_toggled(self):
    self.settings['videoplayer']['safe_margin_action_enabled'] = self.global_panel_interface_videoplayer_safe_margin_action_group.isChecked()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_safe_margin_action_x_margin_changed(self):
    self.settings['videoplayer']['safe_margin_action_x'] = self.global_panel_interface_videoplayer_safe_margin_action_x_margin.value()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_safe_margin_action_y_margin_changed(self):
    self.settings['videoplayer']['safe_margin_action_y'] = self.global_panel_interface_videoplayer_safe_margin_action_y_margin.value()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_safe_margin_action_color_button_clicked(self):
    color = QColorDialog.getColor(options=QColorDialog.ShowAlphaChannel)
    if color.isValid():
        self.settings['videoplayer']['safe_margin_action_color'] = color.name(QColor.HexArgb)
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])
    update_widgets(self)


def global_panel_interface_videoplayer_safe_margin_title_group_toggled(self):
    self.settings['videoplayer']['safe_margin_title_enabled'] = self.global_panel_interface_videoplayer_safe_margin_title_group.isChecked()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_safe_margin_title_x_margin_changed(self):
    self.settings['videoplayer']['safe_margin_title_x'] = self.global_panel_interface_videoplayer_safe_margin_title_x_margin.value()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_safe_margin_title_y_margin_changed(self):
    self.settings['videoplayer']['safe_margin_title_y'] = self.global_panel_interface_videoplayer_safe_margin_title_y_margin.value()
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])


def global_panel_interface_videoplayer_safe_margin_title_color_button_clicked(self):
    color = QColorDialog.getColor(options=QColorDialog.ShowAlphaChannel)
    if color.isValid():
        self.settings['videoplayer']['safe_margin_title_color'] = color.name(QColor.HexArgb)
    self.player_subtitle_layer.update_style(self.settings['videoplayer'])
    update_widgets(self)


def update_widgets(self):
    self.global_panel_interface_videoplayer_fontsize_spinbox.setValue(self.settings['videoplayer'].get('font_size', 40))
    self.global_panel_interface_videoplayer_fontfamily_combobox.setCurrentText(self.settings['videoplayer'].get('font_family', 'Ubuntu'))
    self.global_panel_interface_videoplayer_color_button.setStyleSheet('QPushButton { background-color: ' + self.settings['videoplayer'].get('color', '#ffffffff') + ' }')
    self.global_panel_interface_videoplayer_shadow_group.setChecked(self.settings['videoplayer'].get('shadow_enabled', True))
    self.global_panel_interface_videoplayer_shadow_x_position.setValue(self.settings['videoplayer'].get('shadow_x', 2))
    self.global_panel_interface_videoplayer_shadow_y_position.setValue(self.settings['videoplayer'].get('shadow_y', 2))
    self.global_panel_interface_videoplayer_shadow_color_button.setStyleSheet('QPushButton { background-color: ' + self.settings['videoplayer'].get('shadow_color', '#ff000000') + ' }')
    self.global_panel_interface_videoplayer_backgroundbox_group.setChecked(self.settings['videoplayer'].get('backgroundbox_enabled', True))
    self.global_panel_interface_videoplayer_backgroundbox_padding.setValue(self.settings['videoplayer'].get('backgroundbox_padding', 10))
    self.global_panel_interface_videoplayer_backgroundbox_border_radius.setValue(self.settings['videoplayer'].get('backgroundbox_border_radius', 5))
    self.global_panel_interface_videoplayer_backgroundbox_color_button.setStyleSheet('QPushButton { background-color: ' + self.settings['videoplayer'].get('backgroundbox_color', '#55000000') + ' }')
    self.global_panel_interface_videoplayer_safe_margin_action_group.setChecked(self.settings['videoplayer'].get('safe_margin_action_enabled', False))
    self.global_panel_interface_videoplayer_safe_margin_action_x_margin.setValue(self.settings['videoplayer'].get('safe_margin_action_x', 5))
    self.global_panel_interface_videoplayer_safe_margin_action_y_margin.setValue(self.settings['videoplayer'].get('safe_margin_action_y', 5))
    self.global_panel_interface_videoplayer_safe_margin_action_color_button.setStyleSheet('QPushButton { background-color: ' + self.settings['videoplayer'].get('safe_margin_action_color', '#dd67FF4D') + ' }')
    self.global_panel_interface_videoplayer_safe_margin_title_group.setChecked(self.settings['videoplayer'].get('safe_margin_title_enabled', False))
    self.global_panel_interface_videoplayer_safe_margin_title_x_margin.setValue(self.settings['videoplayer'].get('safe_margin_title_x', 10))
    self.global_panel_interface_videoplayer_safe_margin_title_y_margin.setValue(self.settings['videoplayer'].get('safe_margin_title_y', 10))
    self.global_panel_interface_videoplayer_safe_margin_title_color_button.setStyleSheet('QPushButton { background-color: ' + self.settings['videoplayer'].get('safe_margin_title_color', '#ddff0000') + ' }')




import ctypes
import platform

from PySide6.QtWidgets import QListWidget, QPushButton, QWidget, QVBoxLayout, QFileDialog, QTabWidget, QHBoxLayout, QListView, QLabel, QListWidgetItem, QSizePolicy
from PySide6.QtCore import Signal, Qt, QSize, QRectF, QMargins, QRunnable, QObject
from PySide6.QtGui import QIcon, QPainter, QFont, QColor
from PySide6.QtOpenGLWidgets import QOpenGLWidget


def is_float(element: str) -> bool:
    try:
        float(element)
        return True
    except ValueError:
        return False


def get_timeline_time_str(seconds, ms=False):
    """Function to return timecode from seconds"""
    secs = int(seconds % 60)
    mins = int((seconds / 60) % 60)
    hrs = int((seconds / 60) / 60)
    mss = int(round(float('0.' + str(seconds).split('.', 1)[-1]), 3) * 1000)

    if ms:
        if hrs:
            return "{hh:02d}:{mm:02d}:{ss:02d}.{mss:03d}".format(hh=hrs, mm=mins, ss=secs, mss=mss)
        elif mins:
            return "{mm:02d}:{ss:02d}.{mss:03d}".format(mm=mins, ss=secs, mss=mss)
        else:
            return "{ss:02d}.{mss:03d}".format(ss=secs, mss=mss)
    else:
        if hrs:
            return "{hh:02d}:{mm:02d}:{ss:02d}".format(hh=hrs, mm=mins, ss=secs)
        elif mins:
            return "{mm:02d}:{ss:02d}".format(mm=mins, ss=secs)
        else:
            return "{ss:02d}".format(ss=secs)


def convert_ffmpeg_timecode_to_seconds(timecode):
    """Function to convert ffmpeg timecode to seconds"""
    if timecode:
        final_value = float(timecode.split(':')[-1])
        if timecode.count(':') > 2:
            final_value += int(timecode.split(':')[-2]) * 60.0
        if timecode.count(':') > 3:
            final_value += int(timecode.split(':')[-3]) * 3600.0
        if timecode.count(':') > 4:
            final_value += int(timecode.split(':')[-4]) * 3600.0
        return final_value
    else:
        return False


class GetProcAddressGetter:
    """ fixme: Class gets obsolete once https://bugreports.qt.io/browse/PYSIDE-971 gets fixed """

    def __init__(self):
        self._func = self._find_platform_wrapper()

    def _find_platform_wrapper(self):
        operating_system = platform.system()
        if operating_system == 'Linux':
            return self._init_linux()
        elif operating_system == 'Windows':
            return self._init_windows()
        raise f'Platform {operating_system} not supported yet'

    def _init_linux(self):
        try:
            from OpenGL import GLX
            return self._glx_impl
        except AttributeError:
            pass
        try:
            from OpenGL import EGL
            return self._egl_impl
        except AttributeError:
            pass
        raise 'Cannot initialize OpenGL'

    def _init_windows(self):
        import glfw
        from PySide6.QtGui import QOffscreenSurface, QOpenGLContext

        self.surface = QOffscreenSurface()
        self.surface.create()

        if not glfw.init():
            raise 'Cannot initialize OpenGL'

        glfw.window_hint(glfw.VISIBLE, glfw.FALSE)
        window = glfw.create_window(1, 1, "mpvQC-OpenGL", None, None)

        glfw.make_context_current(window)
        # QOpenGLContext.currentContext().makeCurrent(self.surface)
        return self._windows_impl

    def wrap(self, _, name: bytes):
        address = self._func(name)
        return ctypes.cast(address, ctypes.c_void_p).value

    @staticmethod
    def _glx_impl(name: bytes):
        from OpenGL import GLX
        return GLX.glXGetProcAddress(name.decode("utf-8"))

    @staticmethod
    def _egl_impl(name: bytes):
        from OpenGL import EGL
        return EGL.eglGetProcAddress(name.decode("utf-8"))

    @staticmethod
    def _windows_impl(name: bytes):
        import glfw
        return glfw.get_proc_address(name.decode('utf8'))




"""Subtitles Video panel

"""

import os

from PySide6.QtWidgets import QPushButton, QFileDialog, QWidget

from subtitld.modules.paths import LIST_OF_SUPPORTED_IMPORT_EXTENSIONS
from subtitld.interface import global_panel
from subtitld.modules import file_io


list_of_supported_import_extensions = []
for exttype in LIST_OF_SUPPORTED_IMPORT_EXTENSIONS:
    for ext in LIST_OF_SUPPORTED_IMPORT_EXTENSIONS[exttype]['extensions']:
        list_of_supported_import_extensions.append(ext)


def load_menu(self):
    """Function to load subtitles panel widgets"""
    self.global_panel_import_menu_button = QPushButton('Import')
    self.global_panel_import_menu_button.setCheckable(True)
    self.global_panel_import_menu_button.setProperty('class', 'global_panel_menu')
    self.global_panel_import_menu_button.clicked.connect(lambda: global_panel_menu_changed(self))
    self.global_panel_menu.layout().addWidget(self.global_panel_import_menu_button)


def global_panel_menu_changed(self):
    self.global_panel_import_menu_button.setEnabled(False)
    global_panel.global_panel_menu_changed(self, self.global_panel_import_menu_button, self.global_panel_import_content)


def load_widgets(self):
    """Function to load subtitles panel widgets"""

    self.global_panel_import_content = QWidget()

    self.global_subtitlesvideo_import_button = QPushButton(u'IMPORT', parent=self.global_panel_import_content)
    self.global_subtitlesvideo_import_button.setProperty('class', 'button')
    # self.global_subtitlesvideo_import_button.setCheckable(True)
    self.global_subtitlesvideo_import_button.clicked.connect(lambda: global_subtitlesvideo_import_button_clicked(self))

    # self.global_subtitlesvideo_import_panel = QLabel(parent=self.global_panel_import_content)
    # self.global_subtitlesvideo_import_panel.setVisible(False)

    # self.global_subtitlesvideo_import_panel_radiobox = QRadioBox(parent=self.global_subtitlesvideo_import_panel)
    self.global_panel_content_stacked_widgets.addWidget(self.global_panel_import_content)


def global_subtitlesvideo_import_button_clicked(self):
    """Function to import file"""
    # if self.global_subtitlesvideo_import_button.isChecked():
    #     self.global_subtitlesvideo_export_button.setGeometry(20, 200, self.global_panel_menu.width() - 40, 30)
    # else:
    #     self.global_subtitlesvideo_export_button.setGeometry(20, 120, self.global_panel_menu.width() - 40, 30)
    # self.global_subtitlesvideo_import_panel.setVisible(self.global_subtitlesvideo_import_button.isChecked())

    supported_import_files = self.tr('Text files') + ' ({})'.format(" ".join([" * .{}".format(fo) for fo in list_of_supported_import_extensions]))
    file_to_open = QFileDialog.getOpenFileName(parent=self, caption=self.tr('Select the file to import'), dir=os.path.expanduser("~"), filter=supported_import_files)[0]
    if file_to_open:
        self.subtitles_list += file_io.import_file(filename=file_to_open)[0]
        self.subtitles_list.sort()
        # update_widgets(self)
